#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Aug 30 14:48:40 2019

@author: thomaskallevik
"""
import sys
import Import_script as IM
import numpy as np
import time
import pandas as pd
import pyomo.environ as pyo
from pyomo.opt import SolverFactory
import prob_functions as pf
import Processed_data as PD
import matplotlib.pyplot as plt




"""
i = 0: BaseCase_NVE
i = 1: TOU_NVE
i = 2: CPP
i = 3: RTP_Nordpool
i = 4: EDPR
i = 5: I/C services
i = 6: Measured capacity NVE
i = 7 Subscribed capacity NVE

"""


def Optimization_method():
    
    
    data = IM.Read_from_excel('Input_data_Base_NewTime.xlsx')
    
    #Make a concrete model in Pyomo
    model = pyo.ConcreteModel()
    
    #Create the different sets.
    model.i = pyo.Set(initialize = [0,1,2,3,4,5,6,7], ordered=True) #Demand respons programme

    model.n = pyo.Set(initialize = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29],ordered = True) #Number of parked EV

    model.t = pyo.Set(initialize = [12,13,14,15,16,17,18,19,20,21,22,23,0,1,2,3,4,5,6,7,8,9,10,11], ordered=True) #Time

    model.omega = pyo.Set(initialize =[0,1,2,3,4,5,6,7,8,9]) #Scenarios
    
    
    omega_vekting = {} #This parameter will weight every scenario equal in the objective function
    for omega in model.omega:
        omega_vekting[omega] = 1/len(model.omega)
    
    

    price_spot_tap = {}
    price_spot = {}
    price_spot_nett = {}
    price_spot_tap_rev = {}
    Inc = {} #Different prices for incentives
    Pen = {} #Different prices for penalty
    Capacity_Price = {}
    price_en_adjusted = data['Parameter']['Value']['Price_en_adjusted']

    price_energy_income ={}
    price_reserve = {}
    
    P_Cont = {} # The contract level of power in incentive based programs.
    """
    The different prices to be used in the model, and the constracted power level.
    """
    for i in model.i:
        for t in model.t:
            price_spot_tap[t] = data['Energy_Price']['Price, Energy_Tap'][t]
            price_spot[t] = data['Energy_Price']['Price, Energy'][t]
            price_spot_nett[i,t] = data['En_Price_DRP'][i][t]
            price_spot_tap_rev[t] = data['Energy_Price']['Price_Energy_Tap_-X'][t]
            price_reserve[t] = data['Energy_Price']['Price, Capacity'][t]
            Inc[i,t] = data['Incentive_Price'][i][t]
            Pen[i,t] = data['Penalty'][i][t]
            Capacity_Price[i,t] = data['Capacity_Price'][i][t]
            price_energy_income[t] = data['Energy_Price']['Price, Energy'][t]
            
            P_Cont[t] = data['P_Contract']['P_Cont'][t]
    
    
        
    
    "Create the different parameters"
       
    C_Battery = data['Parameter']['Value']['C_Battery'] #Cost of EV-battery
    L_Et = data['Parameter']['Value']['L_Et'] #Lifetime ev-battery
    Charge_Rate = data['Parameter']['Value']['Charge_Rate'] #Rate of charge ev-battery
    Discharge_Rate = data['Parameter']['Value']['Discharge_Rate'] #Rate of discharge ev-battery
    eff_charge = data['Parameter']['Value']['eff_charge'] #Efficiency of charge ev-battery
    eff_discharge = data['Parameter']['Value']['eff_discharge'] #efficiency of discharge ev-battery
    Degradation = data['Parameter']['Value']['Degradation']
    soc_min_penalty_price = data['Parameter']['Value']['Soc_pen']
    P_Res_Alpha_first = data['Parameter']['Value']['Alpha_first']
    
    
    Gamma_Res = data['Parameter']['Value']['Gamma_Res'] #Penalty if the reserved promised is not met
    FOR_PL = {} #Shows the inability to inject power back to the grid
    agg_percent = {} #The aggregated percentage of the soc that should be met
    
    
    for t in model.t:
        FOR_PL[t] = data['FOR_Penalty']['FOR_PL'][t]
        agg_percent[t] = data['Aggregated_Percentage']['Agg_Percent'][t]
        
    Cap_PEV = {} #The capacity of the different PEV
    for n in model.n:
        Cap_PEV[n] = data['Cap_PEV']['Cap'][n]
    
    
    t_arv = {} #The arrival time for the PEV
    t_dep = {} #The departure time for the PEV
    for omega in model.omega:
        for n in model.n:
            t_arv[omega,n] = data[str(omega)]['Arrive[h]'][n]
            t_dep[omega,n] = data[str(omega)]['Departure[h]'][n]
           
    
    soc_PEV_ini = {} #The soc when car n arrives
    for omega in model.omega:
        for n in model.n:
            soc_PEV_ini[omega,n] = data[str(omega)]['Soc_Pev_ini'][n]
    
    
    
    
    soc_PEV_ini_mellom = {} #Ensure that the PEV only appears in soc_arv when tarv=t
     
    for omega in model.omega:
        for t in model.t:
            for n in model.n:
                soc_PEV_ini_mellom[omega,t,n] = 0
    
    for omega in model.omega:
        for t in model.t:
            for n in model.n:
                if t_arv[omega,n] == t:
                    soc_PEV_ini_mellom[omega,t,n] = soc_PEV_ini[omega,n]


    soc_arv = {} #The total soc added from arriving PEVs in a given hour
    
    for omega in model.omega:
        for t in model.t:
                soc_arv[omega,t] = sum(Cap_PEV[n] * soc_PEV_ini_mellom[omega,t,n] for n in model.n) 
                
            
    
    
    soc_PEV_Sc = {} #soc for the car in the hours where it's present.
    
    for omega in model.omega:
        for t in model.t:
            for n in model.n:
                if t<t_arv[omega,n] and t_dep[omega,n]<t:
                    soc_PEV_Sc[omega,t,n] = 0
                else:
                    soc_PEV_Sc[omega,t,n]= soc_PEV_ini[omega,n]
            
    soc_Sc = {} #The aggregated soc if there where no exchange with the grid.
    for omega in model.omega:
        for t in model.t:
            soc_Sc[omega,t] = sum(Cap_PEV[n] * soc_PEV_Sc[omega,t,n] for n in model.n)
  

    
    N_PEV_arv = {} #Nr of arrived cars in a given hour
    N_PEV_dep = {} #Nr of departed cars in a given hour
    N_PEV_max = data['Parameter']['Value']['N_PEV_MAX'] #Maximal nr of cars
    
    
    for omega in model.omega:        
        for t in model.t:
            N_PEV_arv[omega,t] =0
            N_PEV_dep[omega,t] =0
    
    
    for omega in model.omega:
        for t in model.t:
            for n in model.n:
                if t_arv[omega,n] == t:
                    N_PEV_arv[omega,(t)] +=1
        
                
                
    for omega in model.omega:
        for t in model.t:
            for n in model.n:
                if t_dep[omega,n]==(t):
                    N_PEV_dep[omega,(model.t.next(t))] +=1
        
                
    

    
    #Degradation = C_Battery / L_Et #Cost of degradation
    
    N_PEV = {} #Nr of cars in a given hour
    for omega in model.omega:
        for t in model.t:
            if t==model.t.first():
                N_PEV[omega,t] = N_PEV_arv[omega,t] - N_PEV_dep[omega,t]
            else:
                N_PEV[omega,t] = N_PEV_arv[omega,t] - N_PEV_dep[omega,t] + N_PEV[omega,(model.t.prev(t))]
                
    

    
    soc_PEV_min_car = {} #min soc for car n
    soc_PEV_max_car = {} #max soc for car n
    for n in model.n:
        soc_PEV_min_car[n] = data['SOC_PEV']['soc_min'][n]
        soc_PEV_max_car[n] = data['SOC_PEV']['soc_max'][n]
        
    soc_departure_car = {} #This ensures that car n leaves with the desired soc
    for n in model.n:
        soc_departure_car[n] = data['soc_departure']['soc_dep'][n]
        
        
       
    isparked = {} #boolean value to check if a car is parked or not
    for omega in model.omega:
        for t in model.t:
            for n in model.n:
                if t < t_arv[omega,n] and t_dep[omega,n]<= t:
                    isparked[omega,t,n] = 0
                else:
                    isparked[omega,t,n] = 1
                
    soc_PEV_min_agg = {} #minimum soc of the cars aggregated
    soc_PEV_max_agg = {} #maximum soc of the cars aggregated
    soc_min_departure = {} #soc for the departed cars within an hour
    soc_max_departure = {}
    soc_flex_min_departure = {}
    soc_flex_max_departure = {}
    
    for omega in model.omega:
        for t in model.t:
            soc_PEV_max_agg[omega,t] = sum(data['SOC_PEV']['soc_max'][n]*isparked[omega,t,n] for n in model.n)
            soc_PEV_min_agg[omega,t] = 0
            soc_min_departure[omega,t] = 0
            soc_max_departure[omega,t] = 0
            soc_flex_min_departure[omega,t] =0
            soc_flex_max_departure[omega,t] =0
            
    soc_dep_requirement = data['soc_departure']['Min Sats'][0]
    
    for omega in model.omega:
        for t in model.t:
            for n in model.n:
                if t==11:
                    soc_min_departure[omega,t] += 0
                    soc_max_departure[omega,t] += 0
                    soc_flex_min_departure[omega,t] += 0
                    soc_flex_max_departure[omega,t] += 0
                elif model.t.next(t) ==t_dep[omega,n]:
                    soc_min_departure[omega,t] += soc_departure_car[n]
                    soc_max_departure[omega,t] += soc_PEV_max_car[n]
                    if soc_dep_requirement==0.9:
                        soc_flex_min_departure[omega,t] += 0.4*Cap_PEV[n]
                        soc_flex_max_departure[omega,t] += 0.1*Cap_PEV[n]
                    elif soc_dep_requirement==0.8:
                        soc_flex_min_departure[omega,t] += 0.3*Cap_PEV[n]
                        soc_flex_max_departure[omega,t] += 0.2*Cap_PEV[n]
                    elif soc_dep_requirement==0.7:
                        soc_flex_min_departure[omega,t] += 0.2*Cap_PEV[n]
                        soc_flex_max_departure[omega,t] += 0.3*Cap_PEV[n]

                        
                    

    
    
    for omega in model.omega:
        for t in model.t:
            for n in model.n:
                if t==11:
                    soc_PEV_min_agg[omega,t] +=0
                elif model.t.next(t)==t_dep[omega,n]:
                    soc_PEV_min_agg[omega,t] += soc_departure_car[n]
                else:
                    soc_PEV_min_agg[omega,t] += data['SOC_PEV']['soc_min'][n]*isparked[omega,t,n]
    
    
 

    P_initial = {} #The initial power consumption. Used to check incentive and penalty
    
    for omega in model.omega:
        for t in model.t:
            P_initial[omega,t] = data['P_ini'+ str(omega)]['P_ini'][t]
    
    P_Cont_Real = {} #The power reduction the constract is binding for.
    for omega in model.omega:
        for t in model.t:
            P_Cont_Real[omega,t] = P_initial[omega,t]*P_Cont[t]
    

    
    
    Yearly_fee = {}
    Included_steps_tariff = {}
    
    for i in model.i:
        
        Yearly_fee[i] = data['Fixed_Tariff'][i]['Yearly fee/day']
    
    Included_steps_tariff = data['Parameter']['Value']['Included_steps']/1000
        
        

    
    
                
    
    
    
    
 
    Alpha = {} #The different DRPs get a value
    
    
    Alpha[0] = data['Parameter']['Value']['Program_BaseCase_NVE']#Basecase
    Alpha[1] = data['Parameter']['Value']['Program_TOU_NVE'] #TOU
    Alpha[2] = data['Parameter']['Value']['Program_CPP'] #CPP
    Alpha[3] = data['Parameter']['Value']['Program_RTP'] #RTP
    Alpha[4] = data['Parameter']['Value']['Program_EDPR'] #EDPR
    Alpha[5] = data['Parameter']['Value']['Program_I/C'] #I/C services
    Alpha[6] = data['Parameter']['Value']['Program_Measured_Cap_NVE']
    Alpha[7] = data['Parameter']['Value']['Program_Subscribed_Cap_NVE']
    
    
    
    
    """
    Battery parameters
    """ 
    Capacity_battery = data['Parameter']['Value']['Cap Battery'] #Capacity of battery in kwh/h
    Charge_rate_battery = data['Parameter']['Value']['Charge_Rate_Batt'] #Charge rate for the battery
    Discharge_rate_battery = data['Parameter']['Value']['Charge_Rate_Batt'] #Dischagre rate for the battery
    soc_min_battery = Capacity_battery * data['Parameter']['Value']['Soc_min_battery'] #The minimun soc for the battery
    soc_max_battery = Capacity_battery * data['Parameter']['Value']['Soc_max_battery'] #The maximum soc for the battery
    eff_battery_converter = data['Parameter']['Value']['eff_convert_battery'] #The efficiency for the converter in the battery
    start_soc_battery = Capacity_battery * data['Parameter']['Value']['Start_Soc_Battery'] #The soc when the battery starts
    
   
    """
    Solar parameters
    """
    solar_radiation_ini = {} #The solar radiation collected from xxx
    solar_power_org= {} #The total solar power produced in the parking lot
    solar_power = {} #The solar power injected into the system
    area_parkinglot = data['Parameter']['Value']['area_per_pl'] #Area of one parking lot
    eff_solar = data['Parameter']['Value']['eff_solar'] #The efficiency in the solar panel
    total_area = area_parkinglot * N_PEV_max #The total area of solar panels
    eff_solar_converter = data['Parameter']['Value']['eff_solar_converter'] #Efficiency of the converter
    
    
    for t in model.t: #W/m^2
        solar_radiation_ini[t] = data['Solar_Radiation']['Solar rad'][t]
        
    
    for t in model.t:
        solar_power_org[t] = (solar_radiation_ini[t]/1000000)*eff_solar*total_area
        
    for t in model.t:
        solar_power[t] = solar_power_org[t] * eff_solar_converter
    
    
    
    grid_exchange_rate = 0.1 # N_PEV_max * Charge_Rate + Charge_rate_battery #Maximum power the system can exchange with the grid, either import or export
        
        
        
        
    
        
    #Battery Param
    
    model.Capacity_battery = pyo.Param(initialize = Capacity_battery) #Capacity of battery in kwh/h
    
    model.Charge_rate_battery = pyo.Param(initialize = Charge_rate_battery) #Charge rate for the battery
    
    model.Discharge_rate_battery = pyo.Param(initialize = Discharge_rate_battery) #Dischagre rate for the battery
    
    model.soc_min_battery = pyo.Param(initialize = soc_min_battery) #The minimun soc for the battery
    
    model.soc_max_battery = pyo.Param(initialize = soc_max_battery) #The maximum soc for the battery
    
    model.eff_battery_converter = pyo.Param(initialize = eff_battery_converter) #The efficiency for the converter in the battery
    
    model.start_soc_battery = pyo.Param(initialize = start_soc_battery) #The soc when the battery starts
    
    
    model.grid_exchange_rate = pyo.Param(initialize = grid_exchange_rate) #Maximum power the system can exchange with the grid, either import or export
    
    model.delta_grid = pyo.Var(model.omega, model.i, model.t, within = pyo.Binary) #Only export/import within an hour
    
    
    
    #Battery var
    
    model.soc_battery = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) #soc of the battery
    
    model.P_En_B2G = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) #Energy exported to the grid and the system
    
    model.P_En_G2B = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) #Energy imported to the battery from the grid and the system
    
    model.delta_battery = pyo.Var(model.omega, model.i, model.t, within = pyo.Binary) #Only export/import within an hour
    
    # Solar param
    model.P_En_PV2G = pyo.Param(model.t, initialize = solar_power) # solar power to the system and the grid in kWh/h
    
    

   

    
    """
    Legger til CO2-prising
    """
    
    CO2_values ={} #The amount of CO2 used in a given hour
    CO2_Price = data['Parameter']['Value']['Price_CO2'] #The price for the CO2
    CO2_Penalty_Price = data['Parameter']['Value']['Price_CO2_Penalty'] 
    
    for t in model.t:
        CO2_values[t]=data['CO2_power']['CO2[tonn/MWh]'][t]
        
    
    
    model.CO2_values = pyo.Param(model.t, initialize = CO2_values) #The amount of CO2 used in a given hour
    
    model.CO2_Price = pyo.Param(initialize = CO2_Price) #The price for the CO2
    
    model.Cost_CO2 = pyo.Var(model.omega, model.i, model.t) #The cost added to the objective function for emitting CO2
    
    model.Income_CO2 = pyo.Var(model.omega, model.i, model.t) 
    
    model.CO2_Penalty = pyo.Var(model.omega,model.i, within = pyo.NonNegativeReals)
    
    model.CO2_Penalty_Price = pyo.Param(initialize = CO2_Penalty_Price)
    
    model.CO2_Penalty_Cost = pyo.Var(model.omega, model.i, within = pyo.NonNegativeReals)
    


    "Create the different parameters for the battery system"
    model.Alpha = pyo.Param(model.i, initialize = Alpha ) #The different DRPs get a value
    
    model.omega_vekting = pyo.Param(model.omega, initialize = omega_vekting) #This parameter will weight every scenario equal in the objective function

    #Prices
    model.price_spot_tap = pyo.Param(model.t, initialize = price_spot_tap)
    
    model.price_spot = pyo.Param(model.t, initialize = price_spot)
    
    model.price_spot_nett = pyo.Param(model.i, model.t, initialize = price_spot_nett)
    
    model.price_spot_tap_rev = pyo.Param(model.t, initialize = price_spot_tap_rev)
    
    model.Inc = pyo.Param(model.i, model.t, initialize = Inc) #Different prices for incentives
    
    model.Pen = pyo.Param(model.i, model.t, initialize = Pen) #Different prices for penalty
    
    model.price_en_adjusted = pyo.Param(initialize = price_en_adjusted)
    
    model.price_energy_income = pyo.Param(model.t, initialize = price_energy_income)
    
    model.price_reserve = pyo.Param(model.t, initialize = price_reserve)

    

    
    # PEV parameters
    model.N_PEV = pyo.Param(model.omega, model.t, initialize = N_PEV) #Nr of cars in a given hour
        
    model.N_PEV_arv = pyo.Param(model.omega, model.t, initialize = N_PEV_arv) #Nr of arrived cars in a given hour
    
    model.N_PEV_dep = pyo.Param(model.omega,model.t, initialize = N_PEV_dep) #Nr of departed cars in a given hour
    
    
    
    
    #Energy parameters
    model.P_Cont_Real = pyo.Param(model.omega, model.t, initialize = P_Cont_Real) #The power reduction the constract is binding for.
    
    model.P_Res_Act = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) #Activated power
    
    model.P_initial = pyo.Param(model.omega, model.t, initialize = P_initial) # initial value to be used in Income incentive and Cost penalty
    
    model.P_Res_Act_PL = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Res_Act_B = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Res_Alpha_first = pyo.Param(initialize = P_Res_Alpha_first)
   
   
    #SOC parameters
    
    model.soc_PEV_ini = pyo.Param(model.omega,model.n, initialize = soc_PEV_ini) #The soc when car n arrives
    
    model.soc_Sc = pyo.Param(model.omega,model.t, initialize = soc_Sc) #The aggregated soc if there where no exchange with the grid.
    
    model.soc_PEV_Sc = pyo.Param(model.omega,model.t, model.n, initialize = soc_PEV_Sc) #soc for the car in the hours where it's present.
    
    model.soc_PEV_min_car = pyo.Param(model.n, initialize = soc_PEV_min_car) #min soc for car n
    
    model.soc_PEV_max_car = pyo.Param(model.n, initialize = soc_PEV_max_car) #max soc for car n
    
    model.soc_PEV_min_agg = pyo.Param(model.omega, model.t, initialize = soc_PEV_min_agg) #minimum soc of the cars aggregated
    
    model.soc_PEV_max_agg = pyo.Param(model.omega, model.t, initialize = soc_PEV_max_agg) #maximum soc of the cars aggregated
    
    model.soc_arv = pyo.Param(model.omega, model.t, initialize= soc_arv) #Ensure that the PEV only appears in soc_arv when tarv=t
    
    model.soc_departure_car = pyo.Param(model.n, initialize = soc_departure_car) #This ensures that car n leaves with the desired soc
    
    model.soc_min_departure = pyo.Param(model.omega, model.t, initialize = soc_min_departure)
    
    model.soc_max_departure = pyo.Param(model.omega, model.t, initialize = soc_max_departure)
    
    model.soc_min_penalty_price = pyo.Param(initialize = soc_min_penalty_price)
   
    
    #Battery relevant
    model.C_Battery = pyo.Param(initialize = C_Battery) #Cost of EV-battery
    
    model.L_Et = pyo.Param(initialize = L_Et) #Lifetime ev-battery
    
    model.Charge_Rate = pyo.Param(initialize = Charge_Rate) #Rate of charge ev-battery
    
    model.Discharge_Rate = pyo.Param(initialize = Discharge_Rate) #Rate of discharge ev-battery
    
    model.eff_charge = pyo.Param(initialize = eff_charge) #Efficiency of charge ev-battery
    
    model.eff_discharge = pyo.Param(initialize = eff_discharge) #efficiency of discharge ev-battery
    
    model.Cd = pyo.Param(initialize = Degradation) #Cost of degradation
    
    model.Cost_Battery_Deg = pyo.Var(model.omega, model.i, model.t)
    
    model.CostIncome_battery = pyo.Var(model.omega, model.i)
    



    
    #Other parameters
    model.Gamma_Res = pyo.Param(initialize = Gamma_Res)  #Penalty if the reserved promised is not met
    
    model.FOR_PL = pyo.Param(model.t, initialize = FOR_PL) #Shows the inability to inject power back to the grid
    
    model.Cap_PEV = pyo.Param(model.n, initialize = Cap_PEV) #The capacity of the different PEV
    
    model.agg_percent = pyo.Param(model.t, initialize = agg_percent ) #The aggregated percentage of the soc that should be met
    
    model.t_arv = pyo.Param(model.omega,model.n, initialize = t_arv) #The arrival time for the PEV
    
    model.t_dep = pyo.Param(model.omega,model.n, initialize = t_dep) #The departure time for the PEV

    

    "Create the different variables"
    
    #Objective function variables here-and-now
    model.Income_En_SYS2G = pyo.Var(model.omega, model.i, model.t) 
    
    model.Income_Cap_Res = pyo.Var(model.omega, model.i, model.t) 

    model.Income_En_Tariff = pyo.Var(model.omega, model.i, model.t) 

    model.Cost_En_Tariff = pyo.Var(model.omega, model.i, model.t) 

    model.Cost_En_G2SYS = pyo.Var(model.omega, model.i, model.t) 

    model.Cost_Deg_En = pyo.Var(model.omega, model.i, model.t) 
    
    model.Income_Stay = pyo.Var(model.omega, model.i, model.t) 
    
    model.Income_Inc = pyo.Var(model.omega, model.i, model.t) 
    
    model.Cost_Pen = pyo.Var(model.omega, model.i, model.t) 
    
    model.Cost_soc_min = pyo.Var(model.omega, model.i, model.t)
    
    
    #Objective function variables wait-and-see
    model.Income_Res_Act = pyo.Var(model.omega, model.i, model.t) 
    
    model.Cost_Deg_Res = pyo.Var(model.omega, model.i, model.t) 
    
    model.Cost_Unavailable = pyo.Var(model.omega, model.i, model.t)
    
    model.Cost_Res_Tariff = pyo.Var(model.omega, model.i, model.t) 
    
    
    
    #Energy variables

    model.P_En_PL2G = pyo.Var(model.omega, model.i, model.t, within=pyo.NonNegativeReals) 

    model.P_Res = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 
    
    model.P_En_G2PL = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.Delta_P_En_G2PL = pyo.Var(model.omega, model.i, model.t) 
    
    model.delta_Power_PL = pyo.Var(model.omega,model.i,model.t, within = pyo.Binary)
    
  
    
    
    
    #Energy var for the system ineraction with the grid and power-balance for the system
    model.P_Net_System2G = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals, bounds = (0,0.1))
    
    model.P_Net_G2System = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals, bounds = (0,0.1))
    
    
    
    #Different variables for the soc in the system
    
    
    model.soc_dep = pyo.Var(model.omega,model.i, model.t, within = pyo.NonNegativeReals) 
    
    model.soc = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 
 
    
    model.pen_min_soc = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    
    #Capacity pricing
    
    model.Cost_Capacity = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.Cost_Fixed_Tariff = pyo.Param(model.i, initialize = Yearly_fee, mutable = True)
    
    model.price_capacity = pyo.Param(model.i, model.t, initialize = Capacity_Price)
    
    #model.Yearly_Fee = pyo.Param(model.i, initialize = Yearly_fee)
    
    model.Included_steps_tariff = pyo.Param(initialize = Included_steps_tariff)
    
    model.Capacity_subscribed = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeIntegers)
    
    model.Capacity_Pen = pyo.Var(model.omega,model.i, model.t, within = pyo.NonNegativeIntegers)
    

    """
    Battery
    """
    
    def soc_battery_upper(model,omega,i,t): #The upper bound on the soc for the battery
        return(model.soc_battery[omega,i,t] <= model.soc_max_battery)
    model.soc_battery_upper_const = pyo.Constraint(model.omega, model.i, model.t, rule = soc_battery_upper)
    
    def soc_battery_lower(model,omega,i,t): #The lower bound for the battery's soc
        return(model.soc_min_battery <= model.soc_battery[omega,i,t])
    model.soc_battery_lower_const = pyo.Constraint(model.omega, model.i, model.t, rule = soc_battery_lower)
    
    
    def DischargeRate_battery(model,omega,i,t): #The max dischagre rate for the battery
        return(model.P_En_B2G[omega,i,t] + model.P_Res_Act_B[omega,i,t] <= model.Discharge_rate_battery * model.delta_battery[omega,i,t])
    model.DischargeRate_battery_const = pyo.Constraint(model.omega, model.i, model.t, rule = DischargeRate_battery)

    def ChargeRate_battery(model, omega, i , t): #The max charge rate for the battery
        return(model.P_En_G2B[omega,i,t] <= model.Charge_rate_battery*(1-model.delta_battery[omega,i,t]))
    model.ChargeRate_battery_const = pyo.Constraint(model.omega, model.i, model.t, rule = ChargeRate_battery)
    
    def soc_battery(model,omega,i,t): #the soc for the battery.
        if t==model.t.first():
            return(model.soc_battery[omega,i,t] == model.start_soc_battery + (model.P_En_G2B[omega,i,t] * model.eff_battery_converter) - (model.P_En_B2G[omega,i,t] + model.P_Res_Act_B[omega,i,t]) / model.eff_battery_converter) #model.P_Res_Act[omega,i,t] har lagt til denne
        else:
            return(model.soc_battery[omega,i,t] == model.soc_battery[omega,i,(model.t.prev(t))] + (model.P_En_G2B[omega,i,t] * model.eff_battery_converter) - (model.P_En_B2G[omega,i,t] + model.P_Res_Act_B[omega,i,t])/ model.eff_battery_converter) #model.P_Res_Act[omega,i,t] har lagt til denne
    model.soc_battery_const = pyo.Constraint(model.omega, model.i, model.t, rule = soc_battery)
    
    def soc_battery_aggregated(model,omega,i,t):
        return(model.P_En_B2G[omega,i,t] + model.P_Res_Act_B[omega,i,t] <= model.soc_battery[omega,i,t] * model.agg_percent[t])
    model.soc_battery_aggregated_const = pyo.Constraint(model.omega, model.i, model.t, rule =soc_battery_aggregated)
    
    """
    CO2
    """
    
    #This ensures that the system do not emit CO2
    def CO2_Balance(model,omega,i,t):
        return(sum(model.CO2_values[t] * model.P_Net_System2G[omega,i,t] for t in model.t) + model.CO2_Penalty[omega,i] >= sum(model.CO2_values[t] * model.P_Net_G2System[omega,i,t] for t in model.t) )
    model.CO2_Balance_const = pyo.Constraint(model.omega, model.i, model.t, rule = CO2_Balance)
    

    
    
    """
    Grid constraints. These two constraints ensures that the import and export limit is not exceeded.
    """
    
    def grid_import_limit(model,omega,i,t):
        return(model.P_Net_G2System[omega,i,t] <= model.grid_exchange_rate*model.delta_grid[omega,i,t])
    model.grid_import_limit_const = pyo.Constraint(model.omega, model.i, model.t, rule = grid_import_limit)
    
    def grid_export_limit(model,omega,i,t):
        return(model.P_Net_System2G[omega,i,t] <= model.grid_exchange_rate*(1-model.delta_grid[omega,i,t]))
    model.grid_export_limit_const = pyo.Constraint(model.omega, model.i, model.t, rule = grid_export_limit)
    
    
    """
    Power balance system
    """
    
    #The power balance in the system. The ineraction with the grid, PEV an PV. 
    def Net_G2System(model,omega,i,t):
        return(+ model.P_En_PL2G[omega,i,t]+ model.P_Net_G2System[omega,i,t] + model.P_En_B2G[omega,i,t] + model.P_En_PV2G[t] ==   + model.P_Net_System2G[omega,i,t]  + model.P_En_G2PL[omega,i,t]   + model.P_En_G2B[omega,i,t] ) #
    model.Net_G2System_const = pyo.Constraint(model.omega, model.i, model.t, rule = Net_G2System)
    
    """
    Parking lot
    """
    
    
    #This is the soc for the system with the PEV. It defines the arrived cars, departed and the interaction with the rest of the system. The battery is also included here. 
    def soc(model,omega,i,t):
        if t == model.t.first():
            return(model.soc[omega,i,t] ==  model.soc_arv[omega,t] - model.soc_dep[omega,i,t] + model.P_En_G2PL[omega,i,t] * model.eff_charge - (model.P_En_PL2G[omega,i,t]+ model.P_Res_Act_PL[omega,i,t]) / model.eff_discharge ) # )   
        else:
            return(model.soc[omega,i,t] == model.soc[omega,i,(model.t.prev(t))] + model.soc_arv[omega,t] - model.soc_dep[omega,i,t] + model.P_En_G2PL[omega,i,t] * model.eff_charge - ((model.P_En_PL2G[omega,i,t] + model.P_Res_Act_PL[omega,i,t])/ model.eff_discharge) ) #  
    model.soc_Const = pyo.Constraint(model.omega,model.i,model.t, rule = soc)

    
    def soc_dep_min(model,omega,i,t):
        if t == model.t.first():
            return(model.soc_dep[omega,i,t] ==0)
        else:
            return(model.soc_min_departure[omega,model.t.prev(t)]  <= model.soc_dep[omega,i,t]) #
    model.soc_dep_min_const = pyo.Constraint(model.omega, model.i, model.t, rule = soc_dep_min)
    
    def soc_dep_max(model,omega,i,t):
        if t == model.t.first():
            return(model.soc_dep[omega,i,t] ==0)
        else:
            return(model.soc_dep[omega,i,t] <= model.soc_max_departure[omega,model.t.prev(t)])
    model.soc_dep_max_const = pyo.Constraint(model.omega, model.i, model.t, rule = soc_dep_max)
    
    
    """
    Energy
    """
    
    
    #Gives the difference between what it used originally and what it reduced to in order to meet the incentive-based DRPs
    def delta_P_En_G2PL(model, omega, i, t):
        return(model.Delta_P_En_G2PL[omega,i,t] == (model.P_Net_G2System[omega,i,t] - model.P_initial[omega,t])) 
    model.delta_P_En_G2PL_const = pyo.Constraint(model.omega, model.i, model.t, rule = delta_P_En_G2PL)
    
    """
    P_Res/P_Res_Act
    """
    
    #This constraint says that the amount of reserves that can be provided to grid never can exceed the soc of the system or the max capacity.    
    def P_Res_Energy(model,omega,i,t):
        return(model.P_Res[omega,i,t] <= model.soc[omega,i,t]+ model.soc_battery[omega,i,t] - model.soc_PEV_min_agg[omega,t]- model.soc_min_battery)
    model.P_Res_Energy_const = pyo.Constraint(model.omega, model.i, model.t, rule = P_Res_Energy)
    
    def P_Res_Capacity(model,omega,i,t):
        return(model.P_Res[omega,i,t] <= model.N_PEV[omega,t]*model.Charge_Rate + model.Discharge_rate_battery - model.P_En_PL2G[omega,i,t] - model.P_En_B2G[omega,i,t])  #- model.P_Net_System2G[omega,i,t]
    model.P_Res_Capacity_const = pyo.Constraint(model.omega, model.i, model.t, rule = P_Res_Capacity)
    
    def P_Res_Capacity_Grid(model, omega,i,t):
        return(model.P_Res[omega,i,t] <= (model.grid_exchange_rate*(1-model.delta_grid[omega,i,t]) ) )
    model.P_Res_Capacity_Grid_const = pyo.Constraint(model.omega, model.i, model.t, rule = P_Res_Capacity_Grid)
    
    #Legg denne i excel
    def P_Res_1_step(model,omega,i,t):
        return(model.P_Res_Act[omega,i,t] == model.P_Res[omega,i,t]*model.P_Res_Alpha_first)
    model.P_Res_1_step_const = pyo.Constraint(model.omega, model.i, model.t, rule = P_Res_1_step)
    
    def P_Res_Act(model,omega,i,t):
        return(model.P_Res_Act[omega,i,t] == model.P_Res_Act_PL[omega,i,t] + model.P_Res_Act_B[omega,i,t])
    model.P_Res_Act_const = pyo.Constraint(model.omega, model.i, model.t, rule = P_Res_Act)
    
    
    
    " Define the different parts of the objective function"
    

    
    #Cost of degrading battery
    def Cost_Battery_degradation(model,omega,i,t):
        return(model.Cost_Battery_Deg[omega,i,t] == (model.Cd/2) * model.P_En_B2G[omega,i,t] + (model.Cd/2) * model.P_En_G2B[omega,i,t])
    model.Cost_Battery_degradation_const = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_Battery_degradation)
    
    
    #Gives the rest-value on the battery. 
    def Soc_Left_Battery(model,omega,i,t):
        return(model.soc_battery[omega,i,(model.t.last())] == model.start_soc_battery) #
    model.Soc_Left_Battery_const = pyo.Constraint(model.omega, model.i, model.t, rule = Soc_Left_Battery)
    
    
    
    
    #The penalty for net emitting CO2
    def CO2_Pen_Cost(model,omega,i):
        return(model.CO2_Penalty_Cost[omega,i] == model.CO2_Penalty[omega,i]*model.CO2_Penalty_Price)
    model.CO2_Pen_Cost_const = pyo.Constraint(model.omega, model.i, rule = CO2_Pen_Cost)
    
    
    # This gives the cost of CO2
    def Cost_CO2(model,omega,i,t):
        return(model.Cost_CO2[omega,i,t] == model.CO2_Price * model.CO2_values[t] * model.P_Net_G2System[omega,i,t])
    model.Cost_CO2_Const = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_CO2)
    
    def Income_CO2(model, omega,i,t):
        return(model.Income_CO2[omega,i,t] == model.CO2_Price * model.CO2_values[t] * (model.P_Net_System2G[omega,i,t]))
    model.Income_CO2_const = pyo.Constraint(model.omega, model.i, model.t, rule =Income_CO2)
    
    
    

    
    
    #This is the income from selling energy from the system to the grid
    def Income_En_Sys2G(model,omega,i,t):
        return(model.Income_En_SYS2G[omega,i,t] == model.price_spot_tap[t]* model.P_Net_System2G[omega,i,t])
    model.Income_En_Sys2G_const = pyo.Constraint(model.omega, model.i, model.t, rule = Income_En_Sys2G)
    
    #This is the cost for buying energy from the grid to the system
    def Cost_En_G2Sys(model,omega,i,t):
        return(model.Cost_En_G2SYS[omega,i,t] == model.price_spot_nett[i,t] * model.P_Net_G2System[omega,i,t])
    model.Cost_En_G2Sys_const = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_En_G2Sys)
    

    #Incom from offering its capacity to the system operator
    def Income_Cap_Res(model,omega,i,t):
        return(model.Income_Cap_Res[omega,i,t] == model.price_reserve[t] * model.P_Res[omega,i,t])
    model.Income_En_Res_Const = pyo.Constraint(model.omega, model.i, model.t, rule = Income_Cap_Res)
    
    

    #Cost of degrading EV-battery and external battery
    def Cost_Deg_En(model,omega,i,t): 
        return(model.Cost_Deg_En[omega,i,t] == (model.Cd/2) * model.P_En_PL2G[omega,i,t] + (model.Cd/2) * model.P_En_G2PL[omega,i,t])
    model.Cost_Deg_En_Const = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_Deg_En)
    
    
    
    
    #The income from the incentive-based programs
    def Income_Inc(model,omega,i,t): 
        return(model.Income_Inc[omega,i,t] == model.Delta_P_En_G2PL[omega,i,t] * model.Inc[i,t])
    model.Income_Inc_Const = pyo.Constraint(model.omega, model.i, model.t, rule = Income_Inc)
    
    #The penalty from the incentive-based programs
    def Cost_Pen(model,omega,i,t): 
        return(model.Cost_Pen[omega,i,t] == model.Pen[i,t] * (model.P_Cont_Real[omega,t] - model.Delta_P_En_G2PL[omega,i,t]))
    model.Cost_Pen_Const = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_Pen)
    
    
    #Wait-and-see
    
    #The income from the activated reserves
    def Income_Res_Act(model,omega,i,t):
        return(model.Income_Res_Act[omega,i,t] == model.P_Res_Act[omega,i,t] * model.price_spot_tap[t]) #nett
    model.Income_Res_Act_Const = pyo.Constraint(model.omega, model.i, model.t, rule = Income_Res_Act)
    
    #The cost of degrading the battery with offering the reserves to the grid
    def Cost_Deg_Res(model,omega,i,t): 
        return(model.Cost_Deg_Res[omega,i,t] == model.P_Res_Act[omega,i,t] * model.Cd)
    model.Cost_Deg_Res_Const = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_Deg_Res)
    

    
    #Cost of paying the EV-users back for using their battery
    def Cost_Res_Tariff(model,omega,i,t):
        return(model.Cost_Res_Tariff[omega,i,t] == model.P_Res_Act[omega,i,t] * model.price_spot_tap_rev[t])
    model.Cost_Res_Tariff_Const = pyo.Constraint(model.omega,model.i,model.t, rule = Cost_Res_Tariff)
    
    
    
    
    """
    Remember to include this other places. Include in 2nd stage
    """


   
    """
    def P_Net_G2Sys_Cap(model,omega,i,t):
        if i==6:
            return(model.P_Net_G2System[omega,i,t] ==  model.Capacity_Pen[omega,i,t])
        else:
            return(model.P_Net_G2System[omega,i,t] == model.Capacity_subscribed[omega,i,t] + model.Capacity_Pen[omega,i,t])
    model.P_Net_G2Sys_Cap_const = pyo.Constraint(model.omega, model.i, model.t, rule = P_Net_G2Sys_Cap) 
    """

   
    
 
    #Objective function. Sum up all the hours. Also sum over the different DRPs and scenarios, and they are weighted to the the right amount
    def Objective_function(model):
            Income = sum(sum(sum(model.omega_vekting[omega]*model.Alpha[i]*(model.Income_Inc[omega,i,t]+ model.Income_En_SYS2G[omega,i,t]+ model.Income_Cap_Res[omega,i,t]+ model.Income_CO2[omega,i,t]) for t in model.t) for i in model.i) for omega in model.omega) #Income from the here-and-now            
            Cost = sum(sum( sum(model.omega_vekting[omega]*model.Alpha[i]*( - model.Cost_Pen[omega,i,t]-model.Cost_Battery_Deg[omega,i,t]- model.Cost_CO2[omega,i,t]- model.Cost_Deg_En[omega,i,t]   -model.Cost_En_G2SYS[omega,i,t]  ) for t in model.t  ) for i in model.i ) for omega in model.omega) #Cost from the here-and-now        
            Income2 = sum(sum(sum(model.omega_vekting[omega]*model.Alpha[i]*(model.Income_Res_Act[omega,i,t]) for t in model.t) for i in model.i) for omega in model.omega) #Income from wait and see
            Cost2 =  sum(sum(sum(model.omega_vekting[omega]*model.Alpha[i]*(- model.Cost_Deg_Res[omega,i,t] ) for t in model.t) for i in model.i) for omega in model.omega) #Cost from wait and see -model.Cost_Res_Tariff[omega,i,t]
            
            Cost_CO2_pen = sum(sum(model.omega_vekting[omega]*model.Alpha[i]*-model.CO2_Penalty_Cost[omega,i] for i in model.i) for omega in model.omega)
            Cost_Fixed_Tariff = sum(model.Alpha[i]*-model.Cost_Fixed_Tariff[i] for i in model.i)
            return(  +  Income+Income2+ Cost2 + Cost_CO2_pen+Cost_Fixed_Tariff + Cost  ) #     

    model.objective = pyo.Objective(rule = Objective_function, sense = pyo.maximize)
                
                
    #Constraints

    
    #Ensures that the total ineraction (import) with the rest of the system and the grid do not exceed the limit for the chargers and the battery
    def Charge_Rate(model,omega, i, t):
        return(model.P_En_G2PL[omega,i,t] <= model.N_PEV[omega,t] * model.Charge_Rate * model.delta_Power_PL[omega,i,t] )
    model.Charge_Rate_const = pyo.Constraint(model.omega, model.i, model.t, rule = Charge_Rate)
    
   
    #Ensures that the total (export) with the rest of the system and the grid do not exceed the limit for the chargers and the battery
    def Discharge_Rate(model,omega, i, t):
        return(model.P_En_PL2G[omega,i,t] + model.P_Res_Act_PL[omega,i,t] <= model.N_PEV[omega,t] * model.Discharge_Rate*(1-model.delta_Power_PL[omega,i,t])) #
    model.Discharge_Rate_Const = pyo.Constraint(model.omega, model.i, model.t, rule = Discharge_Rate)
    
    
    #Ensures that the soc for the PEV and the battery always are over min
    def soc_PEV_agg_Smaller(model, omega, i, t):
        return(model.soc_PEV_min_agg[omega,t] <= model.soc[omega,i,t])
    model.soc_PEV_agg_S_const = pyo.Constraint(model.omega, model.i, model.t, rule = soc_PEV_agg_Smaller)
    
    #Ensures that the soc for the PEV and the battery always are under max
    def soc_PEV_agg_Larger(model, omega,i, t):
        return(model.soc[omega,i,t] <= model.soc_PEV_max_agg[omega,t])
    model.soc_PEV_agg_L_const = pyo.Constraint(model.omega, model.i, model.t, rule = soc_PEV_agg_Larger)
    
    
    #Ensures that the exported energy are never over the desired soc-level.
    def Agg(model,omega, i, t):
        return(model.P_En_PL2G[omega,i,t]+ model.P_Res_Act_PL[omega,i,t]   <= model.agg_percent[t] * model.soc[omega,i,t])  # 
    model.agg_const = pyo.Constraint(model.omega, model.i, model.t, rule = Agg)
    
    
    optimization = SolverFactory('gurobi')
    
    
    results = optimization.solve(model, load_solutions = True)
    
    
 

    print("First step=",model.objective())
    
    CO2_pen = 0
   
    for omega in model.omega:
        for i in model.i:
            if 0<Alpha[i]:
                CO2_pen += omega_vekting[omega]*model.CO2_Penalty_Cost[omega,i].value
    print("CO2 Pen in euro=",CO2_pen)
    
    
    
    
    co2 = 0
    for omega in model.omega:
        co2 += omega_vekting[omega]*model.CO2_Penalty[omega,1].value
        for t in model.t:
            co2 += omega_vekting[omega]*model.CO2_values[t] * model.P_Net_G2System[omega,1,t].value
    

    print("CO2 released=", co2)     
    


    
    
    
    #This parts saves the results to an excel-file
    """
    
    

    
    output = {}
    output['Objective'] = {}
    output['Income_En_SYS2G'] = {}
    output['Income_Cap_Res'] = {}
    output['Cost_En_G2SYS'] = {}
    output['Cost_Deg_En'] = {}
    output['Income_Inc'] = {}
    output['Cost_Pen'] = {}
    output['P_En_PL2G'] = {}
    output['P_Res'] = {}
    output['P_En_G2PL'] = {}
    output['Delta_P_En_G2PL'] = {}
    output['soc_dep'] = {}
    output['soc'] = {}
    output['soc_battery'] = {}
    output['P_En_B2G'] = {}
    output['P_En_G2B'] = {}
    output['P_Net_System2G'] = {}
    output['P_Net_G2System'] = {}
    output['Cost_CO2'] = {}
    output['Income_CO2'] = {}
    output['CO2_Penalty'] = {}
    output['CO2_Penalty_Cost'] = {}
    output['delta_Power_PL'] = {}
    output['Cost_Capacity'] = {}
    output['Cost_Fixed_Tariff'] = {}
    output['Capacity_Pen'] = {}
    output['P_Res_Act_PL'] = {}
    output['P_Res_Act_B'] = {}
    output['Cost_Battery_Deg'] = {}
    output['pen_min_soc'] = {}
    

    
    for omega in model.omega:
        for i in model.i:
            for t in model.t:
                if 0 < Alpha[i]:
                    output['Objective'] = model.objective()
                    output['Income_En_SYS2G'][omega,i,t] = model.Income_En_SYS2G[omega,i,t].value
                    output['Income_Cap_Res'][omega,i,t] = model.Income_Cap_Res[omega,i,t].value
                    output['Cost_En_G2SYS'][omega,i,t] = model.Cost_En_G2SYS[omega,i,t].value 
                    output['Cost_Deg_En'][omega,i,t] = model.Cost_Deg_En[omega,i,t].value
                    output['Income_Inc'][omega,i,t] = model.Income_Inc[omega,i,t].value
                    output['Cost_Pen'][omega,i,t]= model.Cost_Pen[omega,i,t].value
                    output['P_En_PL2G'][omega,i,t] = model.P_En_PL2G[omega,i,t].value 
                    output['P_Res'][omega,i,t] = model.P_Res[omega,i,t].value 
                    output['P_En_G2PL'][omega,i,t] = model.P_En_G2PL[omega,i,t].value
                    output['Delta_P_En_G2PL'][omega,i,t] = model.Delta_P_En_G2PL[omega,i,t].value
                    output['soc_dep'][omega,i,t] = model.soc_dep[omega,i,t].value
                    output['soc'][omega,i,t] = model.soc[omega,i,t].value
                    output['soc_battery'][omega,i,t] = model.soc_battery[omega,i,t].value
                    output['P_En_B2G'][omega,i,t] = model.P_En_B2G[omega,i,t].value
                    output['P_En_G2B'][omega,i,t] = model.P_En_G2B[omega,i,t].value
                    output['P_Net_System2G'][omega,i,t] = model.P_Net_System2G[omega,i,t].value
                    output['P_Net_G2System'][omega,i,t] = model.P_Net_G2System[omega,i,t].value 
                    output['Cost_CO2'][omega,i,t] = model.Cost_CO2[omega,i,t].value
                    output['Income_CO2'][omega,i,t] = model.Income_CO2[omega,i,t].value
                    output['CO2_Penalty'][omega,i] = model.CO2_Penalty[omega,i].value
                    output['CO2_Penalty_Cost'][omega,i] = model.CO2_Penalty_Cost[omega,i].value
                    output['delta_Power_PL'][omega,i,t] = model.delta_Power_PL[omega,i,t].value
                    output['Cost_Capacity'][omega,i,t] = model.Cost_Capacity[omega,i,t].value
                    output['Cost_Fixed_Tariff'][i] = model.Cost_Fixed_Tariff[i].value
                    output['Capacity_Pen'][omega,i,t] = model.Capacity_Pen[omega,i,t].value
                    output['P_Res_Act_PL'][omega,i,t] = model.P_Res_Act_PL[omega,i,t].value
                    output['P_Res_Act_B'][omega,i,t] = model.P_Res_Act_B[omega,i,t].value
                    output['Cost_Battery_Deg'][omega,i,t] = model.Cost_Battery_Deg[omega,i,t].value
                    output['pen_min_soc'][omega,i,t] = model.pen_min_soc[omega,i,t].value
                    
    
    resultater = pd.DataFrame(data = output) 

    
    
    #Store each result in an excel file, given a separate sheet
    with pd.ExcelWriter("Results_1step.xlsx") as writer:
        resultater['Objective'].to_excel(writer, sheet_name = "Objective")
        resultater['Income_En_SYS2G'].to_excel(writer, sheet_name = "Income_En_SYS2G")
        resultater['Income_Cap_Res'].to_excel(writer, sheet_name = "Income_Cap_Res")
        resultater['Cost_En_G2SYS'].to_excel(writer, sheet_name = "Cost_En_G2SYS")
        resultater['Cost_Deg_En'].to_excel(writer, sheet_name = "Cost_Deg_En")
        resultater['Income_Inc'].to_excel(writer, sheet_name = "Income_Inc")
        resultater['Cost_Pen'].to_excel(writer, sheet_name = "Cost_Pen")
        resultater['P_En_PL2G'].to_excel(writer, sheet_name = "P_En_PL2G")
        resultater['P_Res'].to_excel(writer, sheet_name = "P_Res")
        resultater['P_En_G2PL'].to_excel(writer, sheet_name = "P_En_G2PL")
        resultater['Delta_P_En_G2PL'].to_excel(writer, sheet_name = "Delta_P_En_G2PL")
        resultater['soc_dep'].to_excel(writer, sheet_name = "soc_dep")
        resultater['soc'].to_excel(writer, sheet_name = "soc")
        resultater['soc_battery'].to_excel(writer, sheet_name = "soc_battery")
        resultater['P_En_B2G'].to_excel(writer, sheet_name = "P_En_B2G")
        resultater['P_En_G2B'].to_excel(writer, sheet_name = "P_En_G2B")
        resultater['P_Net_System2G'].to_excel(writer, sheet_name = "P_Net_System2G")
        resultater['P_Net_G2System'].to_excel(writer, sheet_name = "P_Net_G2System")
        resultater['Cost_CO2'].to_excel(writer, sheet_name = "Cost_CO2")
        resultater['Income_CO2'].to_excel(writer, sheet_name = "Income_CO2")
        resultater['CO2_Penalty'].to_excel(writer, sheet_name = "CO2_Penalty")
        resultater['CO2_Penalty_Cost'].to_excel(writer, sheet_name = "CO2_Penalty_Cost")
        resultater['delta_Power_PL'].to_excel(writer, sheet_name = "delta_Power_PL")
        resultater['Cost_Capacity'].to_excel(writer, sheet_name = "Cost_Capacity")
        resultater['Cost_Fixed_Tariff'].to_excel(writer, sheet_name = "Cost_Fixed_Tariff")
        resultater['Capacity_Pen'].to_excel(writer, sheet_name = "Capacity_Pen")
        resultater['P_Res_Act_PL'].to_excel(writer, sheet_name = "P_Res_Act_PL")
        resultater['P_Res_Act_B'].to_excel(writer, sheet_name = "P_Res_Act_B")
        resultater['Cost_Battery_Deg'].to_excel(writer, sheet_name = "Cost_Battery_Deg")
        resultater['pen_min_soc'].to_excel(writer, sheet_name = "pen_min_soc")


    
    """
    
    
    
    #This parts saves the results to an excel-file
    
    
    output = {}
    output['Objective'] = {}
    output['Income_En_SYS2G'] = {}
    output['Income_Cap_Res'] = {}
    output['Cost_En_G2SYS'] = {}
    output['Cost_Deg_En'] = {}
    output['Income_Inc'] = {}
    output['Cost_Pen'] = {}
    output['P_En_PL2G'] = {}
    output['P_Res'] = {}
    output['P_En_G2PL'] = {}
    output['Delta_P_En_G2PL'] = {}
    output['soc_dep'] = {}
    output['soc'] = {}
    output['soc_battery'] = {}
    output['P_En_B2G'] = {}
    output['P_En_G2B'] = {}
    output['P_Net_System2G'] = {}
    output['P_Net_G2System'] = {}
    output['Cost_CO2'] = {}
    output['Income_CO2'] = {}
    output['CO2_Penalty'] = {}
    output['CO2_Penalty_Cost'] = {}
    output['delta_Power_PL'] = {}
    output['Cost_Capacity'] = {}
    output['Cost_Fixed_Tariff'] = {}
    #output['Capacity_Pen'] = {}
    output['P_Res_Act_PL'] = {}
    output['P_Res_Act_B'] = {}
    output['Cost_Battery_Deg'] = {}
    output['solar power'] = {}
    output['P_Res_Act_First'] = {}
    
    output['Capacity_subscribed'] = {}
    
    output['Income_Res_Act']= {}
    

    output['Objective'] = model.objective()
    
    
    for i in model.i:
        for t in model.t:
            if 0 < Alpha[i]:
                    output['Income_En_SYS2G'][i,t] = 0
                    output['Income_Cap_Res'][i,t] = 0
                    output['Cost_En_G2SYS'][i,t]= 0
                    output['Cost_Deg_En'][i,t] = 0
                    output['Income_Inc'][i,t] = 0
                    output['Cost_Pen'][i,t]= 0
                    #output['P_En_PL2G'][i,t] = 0
                    output['P_Res'][i,t] = 0
                    #output['P_En_G2PL'][i,t] = 0
                    output['Delta_P_En_G2PL'][i,t] = 0
                    output['soc_dep'][i,t] = 0
                    #output['soc'][i,t] = 0
                    #output['soc_battery'][i,t] = 0
                    output['P_En_B2G'][i,t] = 0
                    output['P_En_G2B'][i,t] = 0
                    #output['P_Net_System2G'][i,t] = 0
                    #output['P_Net_G2System'][i,t] = 0
                    output['Cost_CO2'][i,t] = 0
                    output['Income_CO2'][i,t] = 0
                    output['CO2_Penalty'][i] = 0
                    output['CO2_Penalty_Cost'][i] = 0
                    #output['Cost_Capacity'][i,t] = 0
                    output['Cost_Fixed_Tariff'][i] = 0
                    #output['Capacity_Pen'][i,t] = 0
                    output['P_Res_Act_PL'][i,t] = 0
                    output['P_Res_Act_B'][i,t] = 0
                    output['Cost_Battery_Deg'][i,t] = 0
                    output['P_Res_Act_First'][i,t] =0
                     
                   
    for omega in model.omega:    
        for i in model.i:
            for t in model.t:       
            
                if 0 < Alpha[i]:
                    output['Income_En_SYS2G'][i,t] += omega_vekting[omega]* model.Income_En_SYS2G[omega,i,t].value
                    output['Income_Cap_Res'][i,t] += omega_vekting[omega]*model.Income_Cap_Res[omega,i,t].value
                    output['Cost_En_G2SYS'][i,t] += omega_vekting[omega]*model.Cost_En_G2SYS[omega,i,t].value 
                    output['Cost_Deg_En'][i,t] += omega_vekting[omega]*model.Cost_Deg_En[omega,i,t].value
                    output['Income_Inc'][i,t] += omega_vekting[omega]*model.Income_Inc[omega,i,t].value
                    output['Cost_Pen'][i,t]+= omega_vekting[omega]*model.Cost_Pen[omega,i,t].value
                    #output['P_En_PL2G'][i,t] += omega_vekting[omega]*model.P_En_PL2G[omega,i,t].value 
                    output['P_Res'][i,t] += omega_vekting[omega]*model.P_Res[omega,i,t].value 
                    #output['P_En_G2PL'][i,t] +=omega_vekting[omega]* model.P_En_G2PL[omega,i,t].value
                    output['Delta_P_En_G2PL'][i,t] +=omega_vekting[omega]* model.Delta_P_En_G2PL[omega,i,t].value
                    output['soc_dep'][i,t] += omega_vekting[omega]*model.soc_dep[omega,i,t].value
                    #output['soc'][i,t] += omega_vekting[omega]*model.soc[omega,i,t].value
                    #output['soc_battery'][i,t] +=omega_vekting[omega]* model.soc_battery[omega,i,t].value
                    output['P_En_B2G'][i,t] += omega_vekting[omega]*model.P_En_B2G[omega,i,t].value
                    output['P_En_G2B'][i,t] += omega_vekting[omega]*model.P_En_G2B[omega,i,t].value
                    #output['P_Net_System2G'][i,t] +=omega_vekting[omega]* model.P_Net_System2G[omega,i,t].value
                    #output['P_Net_G2System'][i,t] +=omega_vekting[omega]* model.P_Net_G2System[omega,i,t].value 
                    output['Cost_CO2'][i,t] += omega_vekting[omega]*model.Cost_CO2[omega,i,t].value
                    output['Income_CO2'][i,t] +=omega_vekting[omega]* model.Income_CO2[omega,i,t].value
                    
                    output['delta_Power_PL'][omega,i,t] = model.delta_Power_PL[omega,i,t].value
                    #output['Cost_Capacity'][i,t] += omega_vekting[omega]*model.Cost_Capacity[omega,i,t].value
                    output['Cost_Fixed_Tariff'][i] = model.Cost_Fixed_Tariff[i].value
                    #output['Capacity_Pen'][i,t] += omega_vekting[omega]*model.Capacity_Pen[omega,i,t].value
                    output['P_Res_Act_PL'][i,t] += omega_vekting[omega]*model.P_Res_Act_PL[omega,i,t].value
                    output['P_Res_Act_B'][i,t] += omega_vekting[omega]*model.P_Res_Act_B[omega,i,t].value
                    output['Cost_Battery_Deg'][i,t] +=omega_vekting[omega]* model.Cost_Battery_Deg[omega,i,t].value
                    output['P_Res_Act_First'][i,t] += omega_vekting[omega] * model.P_Res_Act[omega,i,t].value
                    output['solar power'][t] = model.P_En_PV2G[t]
                    
                    output['Capacity_subscribed'][omega,i,t] = model.Capacity_subscribed[omega,i,t].value
                    
                    output['soc'][omega,i,t] = model.soc[omega,i,t].value
                    output['P_Net_System2G'][omega,i,t] = model.P_Net_System2G[omega,i,t].value
                    output['P_Net_G2System'][omega,i,t] = model.P_Net_G2System[omega,i,t].value 
                    output['soc_battery'][omega,i,t] = model.soc_battery[omega,i,t].value
                    output['P_En_PL2G'][omega,i,t] = model.P_En_PL2G[omega,i,t].value 
                    output['P_En_G2PL'][omega,i,t] = model.P_En_G2PL[omega,i,t].value
                    output['Income_Res_Act'][omega,i,t] = model.Income_Res_Act[omega,i,t].value
                    
    for i in model.i:
        for omega in model.omega:
            if 0 < Alpha[i]:
                output['CO2_Penalty'][i] += omega_vekting[omega]*model.CO2_Penalty[omega,i].value
                output['CO2_Penalty_Cost'][i] +=omega_vekting[omega]* model.CO2_Penalty_Cost[omega,i].value
    resultater = pd.DataFrame(data = output) 
    

    
    
    #Store each result in an excel file, given a separate sheet
    with pd.ExcelWriter("Results_1step_NewTime_TOU.xlsx") as writer:
        resultater['Objective'].to_excel(writer, sheet_name = "Objective")
        resultater['Income_En_SYS2G'].to_excel(writer, sheet_name = "Income_En_SYS2G")
        resultater['Income_Cap_Res'].to_excel(writer, sheet_name = "Income_Cap_Res")
        resultater['Cost_En_G2SYS'].to_excel(writer, sheet_name = "Cost_En_G2SYS")
        resultater['Cost_Deg_En'].to_excel(writer, sheet_name = "Cost_Deg_En")
        resultater['Income_Inc'].to_excel(writer, sheet_name = "Income_Inc")
        resultater['Cost_Pen'].to_excel(writer, sheet_name = "Cost_Pen")
        resultater['P_En_PL2G'].to_excel(writer, sheet_name = "P_En_PL2G")
        resultater['P_Res'].to_excel(writer, sheet_name = "P_Res")
        resultater['P_En_G2PL'].to_excel(writer, sheet_name = "P_En_G2PL")
        resultater['Delta_P_En_G2PL'].to_excel(writer, sheet_name = "Delta_P_En_G2PL")
        resultater['soc_dep'].to_excel(writer, sheet_name = "soc_dep")
        resultater['soc'].to_excel(writer, sheet_name = "soc")
        resultater['soc_battery'].to_excel(writer, sheet_name = "soc_battery")
        resultater['P_En_B2G'].to_excel(writer, sheet_name = "P_En_B2G")
        resultater['P_En_G2B'].to_excel(writer, sheet_name = "P_En_G2B")
        resultater['P_Net_System2G'].to_excel(writer, sheet_name = "P_Net_System2G")
        resultater['P_Net_G2System'].to_excel(writer, sheet_name = "P_Net_G2System")
        resultater['Cost_CO2'].to_excel(writer, sheet_name = "Cost_CO2")
        resultater['Income_CO2'].to_excel(writer, sheet_name = "Income_CO2")
        resultater['CO2_Penalty'].to_excel(writer, sheet_name = "CO2_Penalty")
        resultater['CO2_Penalty_Cost'].to_excel(writer, sheet_name = "CO2_Penalty_Cost")
        resultater['delta_Power_PL'].to_excel(writer, sheet_name = "delta_Power_PL")
        #resultater['Cost_Capacity'].to_excel(writer, sheet_name = "Cost_Capacity")
        resultater['Cost_Fixed_Tariff'].to_excel(writer, sheet_name = "Cost_Fixed_Tariff")
        #resultater['Capacity_Pen'].to_excel(writer, sheet_name = "Capacity_Pen")
        resultater['P_Res_Act_PL'].to_excel(writer, sheet_name = "P_Res_Act_PL")
        resultater['P_Res_Act_B'].to_excel(writer, sheet_name = "P_Res_Act_B")
        resultater['Cost_Battery_Deg'].to_excel(writer, sheet_name = "Cost_Battery_Deg")
        resultater['P_Res_Act_First'].to_excel(writer, sheet_name = "P_Res_Act_First")
        resultater['solar power'].to_excel(writer, sheet_name = "solar power")
        
        resultater['Capacity_subscribed'].to_excel(writer, sheet_name = "Capacity_subscribed")
        resultater['Income_Res_Act'].to_excel(writer, sheet_name = "Income_Res_Act")
    
    
    """
    Print the data to a graph
    """
    
    """
    SOC for PL
    """

    
    
    omega0=[]
    omega1=[]
    omega2=[]
    omega3=[]
    omega4=[]
    omega5=[]
    omega6=[]
    omega7=[]
    omega8=[]
    omega9=[]
    
    res =[]
    
    for i in model.i:
        for t in model.t:
            if 0 < Alpha[i]:
                omega0.append(resultater['soc'][0,i,t])
                omega1.append(resultater['soc'][1,i,t])
                omega2.append(resultater['soc'][2,i,t])
                omega3.append(resultater['soc'][3,i,t])
                omega4.append(resultater['soc'][4,i,t])
                omega5.append(resultater['soc'][5,i,t])
                omega6.append(resultater['soc'][6,i,t])
                omega7.append(resultater['soc'][7,i,t])
                omega8.append(resultater['soc'][8,i,t])
                omega9.append(resultater['soc'][9,i,t])
    
    res.append(omega0)
    res.append(omega1)
    res.append(omega2)
    res.append(omega3)
    res.append(omega4)
    res.append(omega5)
    res.append(omega6)
    res.append(omega7)
    res.append(omega8)
    res.append(omega9)
    
    
    low = np.percentile(res,0, axis=0 )
    middle = np.percentile(res,50, axis=0 )
    high = np.percentile(res,100, axis=0 )

    tid =["12","13","14","15","16","17","18","19","20","21","22","23","0","1","2","3","4","5","6","7","8","9","10","11"]
    soc_pl = plt.figure()

    plt.plot(tid,low)
    plt.plot(tid,middle)
    plt.plot(tid,high)
   
    plt.legend((' 0% Percentile', ' 50% Percentile', '100% Percentile '), loc='best')  
    plt.ylabel('SOC [MWh]') 
    plt.xlabel('Hour [h]') 
    plt.title("SOC for parking lot")
    plt.grid(True)
    plt.show() 
    soc_pl.savefig('SOC_Parkinglot', dpi=400)   
    
    
    #SOC for battery
    
    
    omega0=[]
    omega1=[]
    omega2=[]
    omega3=[]
    omega4=[]
    omega5=[]
    omega6=[]
    omega7=[]
    omega8=[]
    omega9=[]
    
    res =[]
    
    for i in model.i:
        for t in model.t:
            if 0 < Alpha[i]:
                omega0.append(resultater['soc_battery'][0,i,t])
                omega1.append(resultater['soc_battery'][1,i,t])
                omega2.append(resultater['soc_battery'][2,i,t])
                omega3.append(resultater['soc_battery'][3,i,t])
                omega4.append(resultater['soc_battery'][4,i,t])
                omega5.append(resultater['soc_battery'][5,i,t])
                omega6.append(resultater['soc_battery'][6,i,t])
                omega7.append(resultater['soc_battery'][7,i,t])
                omega8.append(resultater['soc_battery'][8,i,t])
                omega9.append(resultater['soc_battery'][9,i,t])
    
    res.append(omega0)
    res.append(omega1)
    res.append(omega2)
    res.append(omega3)
    res.append(omega4)
    res.append(omega5)
    res.append(omega6)
    res.append(omega7)
    res.append(omega8)
    res.append(omega9)
    
    
    low = np.percentile(res,0, axis=0 )
    middle = np.percentile(res,50, axis=0 )
    high = np.percentile(res,100, axis=0 )

    tid =["12","13","14","15","16","17","18","19","20","21","22","23","0","1","2","3","4","5","6","7","8","9","10","11"]
    soc_b = plt.figure()

    plt.plot(tid,low)
    plt.plot(tid,middle)
    plt.plot(tid,high)
   
    plt.legend()  
    plt.ylabel('SOC [MWh]') 
    plt.xlabel('Hour [h]') 
    plt.title("SOC for battery")
    plt.legend((' 0% Percentile', ' 50% Percentile', '100% Percentile '), loc='best') 
    plt.grid(True)
    plt.show() 
    soc_b.savefig('SOC_Battery', dpi=400)   
    
    
    #Interaction with the grid
    
    
    
    
    
    omega0=[]
    omega1=[]
    omega2=[]
    omega3=[]
    omega4=[]
    omega5=[]
    omega6=[]
    omega7=[]
    omega8=[]
    omega9=[]
    
    res =[]
    
    for i in model.i:
        for t in model.t:
            if 0 < Alpha[i]:
                omega0.append(resultater['P_Net_System2G'][0,i,t]-resultater['P_Net_G2System'][0,i,t])
                omega1.append(resultater['P_Net_System2G'][1,i,t]-resultater['P_Net_G2System'][1,i,t])
                omega2.append(resultater['P_Net_System2G'][2,i,t]-resultater['P_Net_G2System'][2,i,t])
                omega3.append(resultater['P_Net_System2G'][3,i,t]-resultater['P_Net_G2System'][3,i,t])
                omega4.append(resultater['P_Net_System2G'][4,i,t]-resultater['P_Net_G2System'][4,i,t])
                omega5.append(resultater['P_Net_System2G'][5,i,t]-resultater['P_Net_G2System'][5,i,t])
                omega6.append(resultater['P_Net_System2G'][6,i,t]-resultater['P_Net_G2System'][6,i,t])
                omega7.append(resultater['P_Net_System2G'][7,i,t]-resultater['P_Net_G2System'][7,i,t])
                omega8.append(resultater['P_Net_System2G'][8,i,t]-resultater['P_Net_G2System'][8,i,t])
                omega9.append(resultater['P_Net_System2G'][9,i,t]-resultater['P_Net_G2System'][9,i,t])
    
    res.append(omega0)
    res.append(omega1)
    res.append(omega2)
    res.append(omega3)
    res.append(omega4)
    res.append(omega5)
    res.append(omega6)
    res.append(omega7)
    res.append(omega8)
    res.append(omega9)
    
    
    low = np.percentile(res,0, axis=0 )
    middle = np.percentile(res,50, axis=0 )
    high = np.percentile(res,100, axis=0 )

    tid =["12","13","14","15","16","17","18","19","20","21","22","23","0","1","2","3","4","5","6","7","8","9","10","11"]
    s2g = plt.figure()

    plt.plot(tid,low)
    plt.plot(tid,middle)
    plt.plot(tid,high)
   
    plt.legend()  
    plt.ylabel('Power [MWh/h]') 
    plt.xlabel('Hour [h]') 
    plt.title("Interaction system and grid (PEnG2S neg., PEnS2G pos.)")
    plt.legend((' 0% Percentile', ' 50% Percentile', '100% Percentile '), loc='best') 
    plt.grid(True)
    plt.show() 
    s2g.savefig('System_Grid', dpi=400)   
    
    
    
    #Interaction parking lot and system
    
    
    
    
    
    omega0=[]
    omega1=[]
    omega2=[]
    omega3=[]
    omega4=[]
    omega5=[]
    omega6=[]
    omega7=[]
    omega8=[]
    omega9=[]
    
    res =[]
    
    for i in model.i:
        for t in model.t:
            if 0 < Alpha[i]:
                omega0.append(resultater['P_En_PL2G'][0,i,t]-resultater['P_En_G2PL'][0,i,t])
                omega1.append(resultater['P_En_PL2G'][1,i,t]-resultater['P_En_G2PL'][1,i,t])
                omega2.append(resultater['P_En_PL2G'][2,i,t]-resultater['P_En_G2PL'][2,i,t])
                omega3.append(resultater['P_En_PL2G'][3,i,t]-resultater['P_En_G2PL'][3,i,t])
                omega4.append(resultater['P_En_PL2G'][4,i,t]-resultater['P_En_G2PL'][4,i,t])
                omega5.append(resultater['P_En_PL2G'][5,i,t]-resultater['P_En_G2PL'][5,i,t])
                omega6.append(resultater['P_En_PL2G'][6,i,t]-resultater['P_En_G2PL'][6,i,t])
                omega7.append(resultater['P_En_PL2G'][7,i,t]-resultater['P_En_G2PL'][7,i,t])
                omega8.append(resultater['P_En_PL2G'][8,i,t]-resultater['P_En_G2PL'][8,i,t])
                omega9.append(resultater['P_En_PL2G'][9,i,t]-resultater['P_En_G2PL'][9,i,t])
    
    res.append(omega0)
    res.append(omega1)
    res.append(omega2)
    res.append(omega3)
    res.append(omega4)
    res.append(omega5)
    res.append(omega6)
    res.append(omega7)
    res.append(omega8)
    res.append(omega9)
    
    
    low = np.percentile(res,0, axis=0 )
    middle = np.percentile(res,50, axis=0 )
    high = np.percentile(res,100, axis=0 )

    tid =["12","13","14","15","16","17","18","19","20","21","22","23","0","1","2","3","4","5","6","7","8","9","10","11"]
    s2pl = plt.figure()

    plt.plot(tid,low,)
    plt.plot(tid,middle)
    plt.plot(tid,high)
   
    plt.legend()  
    plt.ylabel('Power [MWh/h]') 
    plt.xlabel('Hour [h]') 
    plt.title("Interaction system and parking lot (PEnG2S neg., PEnS2G pos.)")
    plt.legend((' 0% Percentile', ' 50% Percentile', '100% Percentile '), loc='best') 
    plt.grid(True)
    plt.show() 
    s2pl.savefig('System_PL', dpi=400)   

    
    
    
    
                                           
           
    
    

    

    
    
    
    

    
    
    model.objective.deactivate()
    
    
    
    

    P_Res_Alpha_second = data['Parameter']['Value']['Alpha_second']
    
    
    """
    
    Here comes the wait-and-see step for the analysis
    
    
    
    """
    P_En_B2G = {}
    P_En_G2B = {}
    delta_battery ={}
    delta_grid = {}
    Cost_CO2 = {}
    Income_CO2 = {}
    CO2_Penalty = {}
    CO2_Penalty_Cost = {}
    Income_En_PL2G = {}
    Income_Cap_Res = {}
    Cost_En_G2PL = {}
    Cost_Deg_En = {}
    Income_Inc = {}
    Cost_Pen = {}
    P_En_PL2G = {}
    P_Res = {}
    P_En_G2PL = {}
    Delta_P_En_G2PL = {}
    delta_Power_PL = {}
    P_Net_System2G = {}
    P_Net_G2System = {}
    soc_dep = {}
    P_Res_Act = {}
    Cost_Fixed_Tariff = {}
    Cost_Capacity = {}
    Capacity_subscribed = {}
    Capacity_Pen = {}
    
    Income_Sys2Grid = {}
    
    
    
    for omega in model.omega:
        for i in model.i:
            for t in model.t:
                P_En_B2G[omega,i,t] = model.P_En_B2G[omega,i,t].value
                P_En_G2B[omega,i,t] = model.P_En_G2B[omega,i,t].value
                delta_battery[omega,i,t] = model.delta_battery[omega,i,t].value
                delta_grid[omega,i,t] = model.delta_grid[omega,i,t].value
                Cost_CO2[omega,i,t] = model.Cost_CO2[omega,i,t].value
                Income_CO2[omega,i,t] = model.Income_CO2[omega,i,t].value
                CO2_Penalty[omega,i] = model.CO2_Penalty[omega,i].value
                CO2_Penalty_Cost[omega,i] = model.CO2_Penalty_Cost[omega,i].value
                Income_En_PL2G[omega,i,t] = model.Income_En_SYS2G[omega,i,t].value
                Income_Cap_Res[omega,i,t] = model.Income_Cap_Res[omega,i,t].value
                Cost_En_G2PL[omega,i,t] = model.Cost_En_G2SYS[omega,i,t].value
                Cost_Deg_En[omega,i,t] = model.Cost_Deg_En[omega,i,t].value
                Income_Inc[omega,i,t] = model.Income_Inc[omega,i,t].value
                Cost_Pen[omega,i,t] = model.Cost_Pen[omega,i,t].value
                P_En_PL2G[omega,i,t] = model.P_En_PL2G[omega,i,t].value
                P_Res[omega,i,t] = model.P_Res[omega,i,t].value
                P_En_G2PL[omega,i,t] = model.P_En_G2PL[omega,i,t].value
                Delta_P_En_G2PL[omega,i,t] = model.Delta_P_En_G2PL[omega,i,t].value
                delta_Power_PL[omega,i,t] = model.delta_Power_PL[omega,i,t].value
                P_Net_System2G[omega,i,t] = model.P_Net_System2G[omega,i,t].value
                P_Net_G2System[omega,i,t] = model.P_Net_G2System[omega,i,t].value
                
                Income_Sys2Grid[omega,i,t] = model.Income_En_SYS2G[omega,i,t].value


                soc_dep[omega,i,t] = model.soc_dep[omega,i,t].value
                Cost_Fixed_Tariff[i] =model.Cost_Fixed_Tariff[i].value
                Cost_Capacity[omega,i,t] = model.Cost_Capacity[omega,i,t].value
                Capacity_subscribed[omega,i,t] = model.Capacity_subscribed[omega,i,t].value
                Capacity_Pen[omega,i,t] = model.Capacity_Pen[omega,i,t].value
                
               


    #Power parameters and variables
    model.P_En_B2G_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) #Energy exported to the grid and the system
    
    model.P_En_G2B_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) #Energy imported to the battery from the grid and the system
    
    model.P_En_PL2G_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 

    model.P_Res_SP = pyo.Param(model.omega, model.i, model.t, initialize = P_Res) 
    
    model.P_En_G2PL_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.Delta_P_En_G2PL_SP = pyo.Param(model.omega, model.i, model.t, initialize = Delta_P_En_G2PL, mutable=True) 
    
    model.P_Res_Artificial_SP_total = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Res_Artificial_SP_2B = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Res_Artificial_SP_2PL = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Res_Act_SP_B = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Res_Act_SP_PL = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Res_Artificial_SP_PL_TooMuch = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Res_Artificial_SP_B_TooMuch = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Res_Artificial_SP_Total_TooMuch = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Res_Alpha_second = pyo.Param(initialize = P_Res_Alpha_second)
    
    model.P_Res_Act_Paid = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
   
    #Different boolean values
    
    model.delta_battery_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.Binary)
    
    model.delta_grid_SP = pyo.Param(model.omega, model.i, model.t, initialize = delta_grid,  mutable=True)
    
    model.delta_Power_PL_SP = pyo.Var(model.omega,model.i,model.t, within = pyo.Binary)
    
    
    #Objective function here and now
    model.Cost_CO2_SP = pyo.Param(model.omega, model.i, model.t, initialize = Cost_CO2 ) #The cost added to the objective function for emitting CO2
    
    model.Income_CO2_SP = pyo.Var(model.omega, model.i, model.t) 
    
    model.CO2_Penalty_SP = pyo.Var(model.omega,model.i, within = pyo.NonNegativeReals )
    
    model.CO2_Penalty_Cost_SP = pyo.Var(model.omega, model.i, within = pyo.NonNegativeReals)
    
    model.Income_En_PL2G_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 
    
    model.Income_Cap_Res_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 

    model.Income_En_Tariff_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 

    model.Cost_En_Tariff_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 

    model.Cost_En_G2PL_SP = pyo.Var(model.omega, model.i, model.t,within = pyo.NonNegativeReals) 

    model.Cost_Deg_En_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 
    
    model.Cost_soc_min_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 
    
    
    model.Income_Inc_SP = pyo.Param(model.omega, model.i, model.t, initialize = Income_Inc, mutable=True) 
    
    model.Cost_Pen_SP = pyo.Param(model.omega, model.i, model.t, initialize = Cost_Pen, mutable=True) 
    
    model.Cost_P_Res_Artificial_SP_total = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.Cost_P_Res_Art_TooMuch_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    

    
    model.Cost_Battery_Deg_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    
    
    #Objective function variables wait-and-see
    model.Income_Res_Act_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 
    
    model.Cost_Deg_Res_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 
    
    model.Cost_Unavailable_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.Cost_Res_Tariff_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 
    
    

    
    #Energy var for the system ineraction with the grid and power-balance for the system
    model.P_Net_System2G_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.P_Net_G2System_SP = pyo.Param(model.omega, model.i, model.t, initialize =P_Net_G2System, mutable=True )
    
    model.P_Net_System2G_SP_Fixed = pyo.Param(model.omega, model.i, model.t, initialize = P_Net_System2G, mutable=True )
    
    
    
    
    #Different variables for the soc in the system

    
    model.soc_dep_SP = pyo.Var(model.omega,model.i, model.t, within = pyo.NonNegativeReals) #
    
    model.soc_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) 
    
    model.soc_battery_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals) #soc of the battery
    
    model.pen_min_soc_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.pen_max_soc_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.pen_limit_soc_min_SP = pyo.Param(model.omega, model.t, initialize = soc_flex_min_departure)
    
    model.pen_limit_soc_max_SP = pyo.Param(model.omega, model.t, initialize = soc_flex_max_departure)
    
    
    #Capacity
    model.Cost_Capacity_SP = pyo.Var(model.omega, model.i, model.t, within = pyo.NonNegativeReals)
    
    model.Cost_Fixed_Tariff_SP = pyo.Param(model.i, initialize = Yearly_fee, mutable = True) 
    
    model.Capacity_subscribed_SP = pyo.Var(model.omega,model.i, model.t, within = pyo.NonNegativeIntegers)
    
    model.Capacity_Pen_SP = pyo.Var(model.omega,model.i, model.t, within = pyo.NonNegativeIntegers)
    
    #Define P_Res_Act from step one
    for omega in model.omega:
        for i in model.i:
            for t in model.t:
                P_Res_Act[omega,i,t] = model.P_Res_SP[omega,i,t] * model.P_Res_Alpha_second
    
    model.P_Res_Act_SP = pyo.Param(model.omega, model.i, model.t, initialize = P_Res_Act)
    
    


    """
    Objective function
    """
    

    def Pen_soc_min_SP(model, omega, i, t):
        return(model.Cost_soc_min_SP[omega,i,t] == (model.pen_min_soc_SP[omega,i,t] + model.pen_max_soc_SP[omega,i,t])*0.001)   #  
    model.Pen_soc_min_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Pen_soc_min_SP)
    

    #Gives the rest-value on the battery.
    def Soc_Left_Battery_SP(model,omega,i,t):
        return(model.soc_battery_SP[omega,i,model.t.last()] == model.start_soc_battery)
    model.Soc_Left_Battery_SP_const = pyo.Constraint(model.omega, model.i, model.t, rule = Soc_Left_Battery_SP)
    
    
    def Cost_Battery_degradation_SP(model,omega,i,t):
        return(model.Cost_Battery_Deg_SP[omega,i,t] == (model.Cd/2) * model.P_En_B2G_SP[omega,i,t] + (model.Cd/2) * model.P_En_G2B_SP[omega,i,t] )
    model.Cost_Battery_degradation_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_Battery_degradation_SP)
    
    """
    #The income from the incentive-based programs
    def Income_Inc_SP(model,omega,i,t): 
        return(model.Income_Inc_SP[omega,i,t] == model.Delta_P_En_G2PL_SP[omega,i,t] * model.Inc[i,t])
    model.Income_Inc_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Income_Inc_SP)
    
    #The penalty from the incentive-based programs
    def Cost_Pen_SP(model,omega,i,t): 
        return(model.Cost_Pen_SP[omega,i,t] == model.Pen[i,t] * (model.P_Cont_Real[omega,t] - model.Delta_P_En_G2PL_SP[omega,i,t]))
    model.Cost_Pen_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_Pen_SP)
    """

    

    
    #This is the income from selling energy from the system to the grid
    def Income_En_Sys2G_SP(model,omega,i,t):
        return(model.Income_En_PL2G_SP[omega,i,t] == P_Net_System2G[omega,i,t]*model.price_spot_tap[t] )
    model.Income_En_Sys2G_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Income_En_Sys2G_SP)
    
    #This is the cost for buying energy from the grid to the system
    def Cost_En_G2Sys_SP(model,omega,i,t):
        return(model.Cost_En_G2PL_SP[omega,i,t] == price_spot_nett[i,t] * model.P_Net_G2System_SP[omega,i,t])
    model.Cost_En_G2Sys_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_En_G2Sys_SP)
    

    #Incom from offering its capacity to the system operator
    def Income_Cap_Res_SP(model,omega,i,t):
        return(model.Income_Cap_Res_SP[omega,i,t] == model.price_reserve[t] * model.P_Res_SP[omega,i,t])
    model.Income_En_Res_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Income_Cap_Res_SP)
    
    

    #Cost of degrading EV-battery and external battery
    def Cost_Deg_En_SP(model,omega,i,t): 
        return(model.Cost_Deg_En_SP[omega,i,t] == (model.Cd/2) * model.P_En_PL2G_SP[omega,i,t] + (model.Cd/2) * model.P_En_G2PL_SP[omega,i,t])
    model.Cost_Deg_En_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_Deg_En_SP)
    
    
    def Cost_CO2_SP(model,omega,i,t):
        return(model.Cost_CO2_SP[omega,i,t] == model.CO2_Price * model.CO2_values[t] * model.P_Net_G2System_SP[omega,i,t])
    model.Cost_CO2_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_CO2_SP)
    
    def Income_CO2_SP(model, omega,i,t):
        return(model.Income_CO2_SP[omega,i,t] == model.CO2_Price * model.CO2_values[t] * (model.P_Net_System2G_SP[omega,i,t])) #model.P_Res_Act_SP[omega,i,t]-model.P_Res_Artificial_SP_total[omega,i,t]
    model.Income_CO2_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule =Income_CO2_SP)
    
    def CO2_Pen_Cost_SP(model,omega,i):
        return(model.CO2_Penalty_Cost_SP[omega,i] == model.CO2_Penalty_SP[omega,i]*model.CO2_Penalty_Price)
    model.CO2_Pen_Cost_const_SP = pyo.Constraint(model.omega, model.i, rule = CO2_Pen_Cost_SP)
    
    #Wait-and-see
    
    #The income from the activated reserves
    def Income_Res_Act_SP(model,omega,i,t):
        return(model.Income_Res_Act_SP[omega,i,t] == model.P_Res_Act_Paid[omega,i,t] * model.price_spot_tap[t])
    model.Income_Res_Act_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Income_Res_Act_SP)
    
    #The cost of degrading the battery with offering the reserves to the grid
    def Cost_Deg_Res_SP(model,omega,i,t): 
        return(model.Cost_Deg_Res_SP[omega,i,t] == model.P_Res_Act_Paid[omega,i,t] * model.Cd)
    model.Cost_Deg_Res_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Cost_Deg_Res_SP)
    
    #Cost of not being able to provide the promised amount.
    def Cost_Unavailable_SP(model,omega,i,t):
        return(model.Cost_Unavailable_SP[omega,i,t] == model.P_Res_Act_Paid[omega,i,t] * model.price_spot_nett[i,t] * model.Gamma_Res * model.FOR_PL[t])
    model.Cost_Unavailable_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule =Cost_Unavailable_SP)
    
    """
    #Cost of paying the EV-users back for using their battery
    def Cost_Res_Tariff_SP(model,omega,i,t):
        return(model.Cost_Res_Tariff_SP[omega,i,t] == model.P_Res_Act_SP[omega,i,t]* model.price_spot_tap_rev[t])
    model.Cost_Res_Tariff_Const_SP = pyo.Constraint(model.omega,model.i,model.t, rule = Cost_Res_Tariff_SP)
    """
    

    
    
    """
    SOC for parking lot
    """
    #This is the soc for the system with the PEV. It defines the arrived cars, departed and the interaction with the rest of the system. The battery is also included here. 
    def soc_SP(model,omega,i,t):
        if t == model.t.first():
            return(model.soc_SP[omega,i,t] ==  model.soc_arv[omega,t] - model.soc_dep_SP[omega,i,t] + model.P_En_G2PL_SP[omega,i,t] * model.eff_charge - ((model.P_En_PL2G_SP[omega,i,t]+ model.P_Res_Act_SP_PL[omega,i,t] ) / model.eff_discharge)  ) # )  
        else:
            return(model.soc_SP[omega,i,t] == model.soc_SP[omega,i,(model.t.prev(t))] + model.soc_arv[omega,t] - model.soc_dep_SP[omega,i,t] + model.P_En_G2PL_SP[omega,i,t] * model.eff_charge - ((model.P_En_PL2G_SP[omega,i,t] + model.P_Res_Act_SP_PL[omega,i,t]) / model.eff_discharge)   ) # 
    model.soc_Const_SP = pyo.Constraint(model.omega,model.i,model.t, rule = soc_SP)
    
    
    
    def soc_dep_min_SP(model,omega,i,t):
        if t == model.t.first():
            return pyo.Constraint.Skip
        else:
            return(model.soc_min_departure[omega,model.t.prev(t)]  - model.pen_min_soc_SP[omega,i,t]  <= model.soc_dep_SP[omega,i,t]) #
    model.soc_dep_min_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = soc_dep_min_SP)
    
    def soc_dep_max_SP(model,omega,i,t):
        if t == model.t.first():
            return pyo.Constraint.Skip
        else:
            return(model.soc_dep_SP[omega,i,t] <= model.soc_max_departure[omega,model.t.prev(t)] + model.pen_max_soc_SP[omega,i,t] )   #
    model.soc_dep_max_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = soc_dep_max_SP)
    
    def soc_dep_pen_limit_low(model,omega,i,t):
        if t == model.t.first():
            return(model.pen_min_soc_SP[omega,i,t] ==0)
        else:
            return(model.pen_min_soc_SP[omega,i,t] <= model.pen_limit_soc_min_SP[omega,model.t.prev(t)])
    model.soc_dep_pen_limit_low_const = pyo.Constraint(model.omega,model.i, model.t, rule = soc_dep_pen_limit_low)
    
    def soc_dep_pen_limit_high(model,omega,i,t):
        if t == model.t.first():
            return(model.pen_max_soc_SP[omega,i,t] ==0)
        else:
            return(model.pen_max_soc_SP[omega,i,t] <= model.pen_limit_soc_max_SP[omega,model.t.prev(t)])
    model.soc_dep_pen_limit_high_const = pyo.Constraint(model.omega,model.i, model.t, rule = soc_dep_pen_limit_high)
    

    
    """
    Battery 
    """
    
    def DischargeRate_battery_SP(model,omega,i,t): #The max dischagre rate for the battery
        return(model.P_En_B2G_SP[omega,i,t] + model.P_Res_Act_SP_B[omega,i,t] <= model.Discharge_rate_battery * model.delta_battery_SP[omega,i,t])
    model.DischargeRate_battery_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = DischargeRate_battery_SP)
    
    def ChargeRate_battery_SP(model, omega, i , t): #The max charge rate for the battery
        return(model.P_En_G2B_SP[omega,i,t] <= model.Charge_rate_battery*(1-model.delta_battery_SP[omega,i,t]))
    model.ChargeRate_battery_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = ChargeRate_battery_SP)
    
    def soc_battery_upper_SP(model,omega,i,t): #The upper bound on the soc for the battery
        return(model.soc_battery_SP[omega,i,t] <= model.soc_max_battery)
    model.soc_battery_upper_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = soc_battery_upper_SP)
    
    def soc_battery_lower_SP(model,omega,i,t): #The lower bound for the battery's soc
        return(model.soc_min_battery <= model.soc_battery_SP[omega,i,t])
    model.soc_battery_lower_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = soc_battery_lower_SP)
    
    
    def soc_battery_SP(model,omega,i,t): #the soc for the battery.
        if t==model.t.first():
            return(model.soc_battery_SP[omega,i,t] == model.start_soc_battery + (model.P_En_G2B_SP[omega,i,t] * model.eff_battery_converter) - (model.P_En_B2G_SP[omega,i,t] + model.P_Res_Act_SP_B[omega,i,t] ) / model.eff_battery_converter ) #model.P_Res_Act[omega,i,t] har lagt til denne
        else:
            return(model.soc_battery_SP[omega,i,t] == model.soc_battery_SP[omega,i,(model.t.prev(t))] + (model.P_En_G2B_SP[omega,i,t] * model.eff_battery_converter) - (model.P_En_B2G_SP[omega,i,t] + model.P_Res_Act_SP_B[omega,i,t] ) / model.eff_battery_converter) #model.P_Res_Act[omega,i,t] har lagt til denne
    model.soc_battery_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = soc_battery_SP)
    
    def soc_battery_aggregated_SP(model,omega,i,t):
        return(model.P_En_B2G_SP[omega,i,t] + model.P_Res_Act_SP_B[omega,i,t]  <= model.soc_battery_SP[omega,i,t] * model.agg_percent[t])
    model.soc_battery_aggregated_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule =soc_battery_aggregated_SP)
    
    """
    CO2
    """
    
    
    def CO2_Balance_SP(model,omega,i,t):
        return(sum(model.CO2_values[t] * model.P_Net_System2G_SP[omega,i,t] for t in model.t) + model.CO2_Penalty_SP[omega,i] >= sum(model.CO2_values[t] * model.P_Net_G2System_SP[omega,i,t] for t in model.t))
    model.CO2_Balance_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = CO2_Balance_SP)
    
    """
    Grid definitions
    """
    
    def grid_import_limit_SP(model,omega,i,t):
        return(model.P_Net_G2System_SP[omega,i,t] <= model.grid_exchange_rate * model.delta_grid_SP[omega,i,t])
    model.grid_import_limit_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = grid_import_limit_SP)
    
    
    def grid_export_limit_upper_SP(model,omega,i,t):
        return(model.P_Net_System2G_SP[omega,i,t] <= (model.P_Net_System2G_SP_Fixed[omega,i,t] + model.P_Res_Act_SP[omega,i,t] - model.P_Res_Artificial_SP_total[omega,i,t])*(1-model.delta_grid_SP[omega,i,t]))
    model.grid_export_limit_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = grid_export_limit_upper_SP)
    
    
    def grid_export_limit_lower_SP(model,omega,i,t):
        return(model.P_Net_System2G_SP_Fixed[omega,i,t]*(1-model.delta_grid_SP[omega,i,t]) <= model.P_Net_System2G_SP[omega,i,t])
    model.grid_export_limit_lower_SP_const = pyo.Constraint(model.omega, model.i, model.t, rule = grid_export_limit_lower_SP)
    
    
    def grid_export_limit_SP(model,omega,i,t):
        return(model.P_Net_System2G_SP[omega,i,t] <= model.grid_exchange_rate*(1-model.delta_grid_SP[omega,i,t]))
    model.grid_export_limit_SP = pyo.Constraint(model.omega, model.i, model.t, rule = grid_export_limit_SP)
    
    
    #The power balance in the system. The ineraction with the grid, PEV an PV. 
    def Net_G2System_SP(model,omega,i,t):
        return(+ model.P_En_PL2G_SP[omega,i,t]+ model.P_Net_G2System_SP[omega,i,t] + model.P_En_B2G_SP[omega,i,t] + model.P_En_PV2G[t]  ==   + model.P_Net_System2G_SP[omega,i,t]  + model.P_En_G2PL_SP[omega,i,t]   + model.P_En_G2B_SP[omega,i,t] ) #
    model.Net_G2System_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Net_G2System_SP)
    
    

    """
    Reserve market definitons
    """
    
    
    #This constraint says that the amount of reserves that can be provided to grid never can exceed the soc of the system or the max capacity.  
    

    
    
    def P_Res_Artificial_Value_SP(model,omega,i,t):
        return(model.P_Res_Artificial_SP_total[omega,i,t] == model.P_Res_Artificial_SP_2B[omega,i,t] + model.P_Res_Artificial_SP_2PL[omega,i,t])
    model.P_Res_Artificial_Value_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = P_Res_Artificial_Value_SP)
    
    
    def P_Res_Art_limit_SP(model,omega,i,t):
        return(model.P_Res_Artificial_SP_total[omega,i,t] <= model.P_Res_Act_SP[omega,i,t])
    model.P_Res_Art_limit_SP_const = pyo.Constraint(model.omega, model.i, model.t, rule = P_Res_Art_limit_SP)
    
    def P_Res_Artificial_SP(model,omega,i,t):
        return(model.Cost_P_Res_Artificial_SP_total[omega,i,t] == model.P_Res_Artificial_SP_total[omega,i,t]*price_reserve[t])
    model.P_Res_Artificial_const_SP = pyo.Constraint(model.omega,model.i,model.t, rule = P_Res_Artificial_SP)
    
    def P_Res_Act_Total(model,omega,i,t):
        return(model.P_Res_Act_SP[omega,i,t] == model.P_Res_Act_SP_B[omega,i,t] + model.P_Res_Act_SP_PL[omega,i,t] + model.P_Res_Artificial_SP_Total_TooMuch[omega,i,t] + model.P_Res_Artificial_SP_total[omega,i,t]) #
    model.P_Res_Act_SP_const = pyo.Constraint(model.omega,model.i, model.t, rule = P_Res_Act_Total)
    
    def P_Res_Act_Paid(model,omega,i,t):
        return(model.P_Res_Act_Paid[omega,i,t] == model.P_Res_Act_SP_B[omega,i,t] + model.P_Res_Act_SP_PL[omega,i,t])
    model.P_Res_Act_Paid_const = pyo.Constraint(model.omega, model.i, model.t, rule = P_Res_Act_Paid)
    
    def P_Res_Act_Total_TooMuch_SP(model, omega,i,t):
        return(model.P_Res_Artificial_SP_Total_TooMuch[omega,i,t] == model.P_Res_Artificial_SP_B_TooMuch[omega,i,t] + model.P_Res_Artificial_SP_PL_TooMuch[omega,i,t])
    model.P_Res_Act_Total_TooMuch_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = P_Res_Act_Total_TooMuch_SP)
    
    def P_Res_Act_Total_TooMuch_Cost_SP(model, omega, i, t):
        return(model.Cost_P_Res_Art_TooMuch_SP[omega,i,t] == model.P_Res_Artificial_SP_Total_TooMuch[omega,i,t]*price_reserve[t])
    model.P_Res_Act_Total_TooMuch_Cost_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = P_Res_Act_Total_TooMuch_Cost_SP)
    
    

    """
    #Gives the difference between what it used originally and what it reduced to in order to meet the incentive-based DRPs
    def delta_P_En_G2PL_SP(model, omega, i, t):
        return(model.Delta_P_En_G2PL_SP[omega,i,t] == (model.P_Net_G2System_SP[omega,i,t] - model.P_initial[omega,t])) 
    model.delta_P_En_G2PL_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = delta_P_En_G2PL_SP)
    """
    
   

    

    

 
  
   
    model.objective.deactivate()
    
 
    #Objective function. Sum up all the hours. Also sum over the different DRPs and scenarios, and they are weighted to the the right amount
    def Objective_function_SP(model):
            Income_SP = sum(sum(sum(model.omega_vekting[omega]*model.Alpha[i]*( model.Income_Inc_SP[omega,i,t]+ model.Income_En_PL2G_SP[omega,i,t]+ model.Income_Cap_Res_SP[omega,i,t] + model.Income_CO2_SP[omega,i,t]) for t in model.t) for i in model.i) for omega in model.omega) #Income from the here-and-now            
            Cost_SP = sum(sum(sum(model.omega_vekting[omega]*model.Alpha[i]*( - model.Cost_Pen_SP[omega,i,t]  -model.Cost_Battery_Deg_SP[omega,i,t]- model.Cost_CO2_SP[omega,i,t]- model.Cost_Deg_En_SP[omega,i,t] - model.Cost_En_G2PL_SP[omega,i,t]   ) for t in model.t  ) for i in model.i ) for omega in model.omega) #Cost from the here-and-now             
            Income2_SP = sum(sum(sum(model.omega_vekting[omega]*model.Alpha[i]*(model.Income_Res_Act_SP[omega,i,t]) for t in model.t) for i in model.i) for omega in model.omega) #Income from wait and see
            Cost2_SP =  sum(sum(sum(model.omega_vekting[omega]*model.Alpha[i]*(- model.Cost_soc_min_SP[omega,i,t]-model.Cost_P_Res_Artificial_SP_total[omega,i,t]- model.Cost_Deg_Res_SP[omega,i,t] - model.Cost_Unavailable_SP[omega,i,t]-model.Cost_P_Res_Art_TooMuch_SP[omega,i,t]) for t in model.t) for i in model.i) for omega in model.omega) #Cost from wait and see   -model.Cost_Res_Tariff_SP[omega,i,t]
            
            Cost_CO2_pen_SP = sum(sum(model.omega_vekting[omega]*model.Alpha[i]*-model.CO2_Penalty_Cost_SP[omega,i] for i in model.i) for omega in model.omega)
            Cost_Fixed_Tariff_SP = sum(model.Alpha[i]*-model.Cost_Fixed_Tariff_SP[i] for i in model.i)
            return(+ Income_SP + Cost_CO2_pen_SP+Cost_Fixed_Tariff_SP   + Income2_SP+ Cost2_SP+ Cost_SP   ) #   

    model.objective_SP = pyo.Objective(rule = Objective_function_SP, sense = pyo.maximize)
                
                
    #Constraints
    

    
    
     #Ensures that the total ineraction (import) with the rest of the system and the grid do not exceed the limit for the chargers and the battery
    def Charge_Rate_SP(model,omega, i, t):
        return(model.P_En_G2PL_SP[omega,i,t] <= model.N_PEV[omega,t] * model.Charge_Rate * model.delta_Power_PL_SP[omega,i,t] )
    model.Charge_Rate_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Charge_Rate_SP)
    
    
    #Ensures that the total (export) with the rest of the system and the grid do not exceed the limit for the chargers and the battery
    def Discharge_Rate_SP(model,omega, i, t):
        return(model.P_En_PL2G_SP[omega,i,t] + model.P_Res_Act_SP_PL[omega,i,t] <= model.N_PEV[omega,t] * model.Discharge_Rate*(1-model.delta_Power_PL_SP[omega,i,t])) #
    model.Discharge_Rate_Const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Discharge_Rate_SP)
    
    
    
    #Ensures that the soc for the PEV and the battery always are over min
    def soc_PEV_agg_Smaller_SP(model, omega, i, t):
        return(model.soc_PEV_min_agg[omega,t] - model.pen_min_soc_SP[omega,i,t] <= model.soc_SP[omega,i,t])  #
    model.soc_PEV_agg_S_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = soc_PEV_agg_Smaller_SP)
    
    
    #Ensures that the soc for the PEV and the battery always are under max
    def soc_PEV_agg_Larger_SP(model, omega,i, t):
        return(model.soc_SP[omega,i,t] <= model.soc_PEV_max_agg[omega,t] + model.pen_max_soc_SP[omega,i,t])  #
    model.soc_PEV_agg_L_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = soc_PEV_agg_Larger_SP)
    
    
    #Ensures that the exported energy are never over the desired soc-level.
    def Agg_SP(model,omega, i, t):
        return(model.P_En_PL2G_SP[omega,i,t]+ model.P_Res_Act_SP_PL[omega,i,t]    <= model.agg_percent[t] * model.soc_SP[omega,i,t])  # 
    model.agg_const_SP = pyo.Constraint(model.omega, model.i, model.t, rule = Agg_SP)
    
    
    optimization = SolverFactory('gurobi')
    
    
    results_SP = optimization.solve(model, load_solutions = True)     

    """
    
    for omega in model.omega:
        for t in model.t:
            
        
            print(t,model.Income_Res_Act_SP[0,1,t].value)
    """
    print("Second step=",model.objective_SP())
    
    CO2_pen = 0
    SO_pen = 0
    SOC_pen= 0
    for omega in model.omega:
        for i in model.i:
            if 0<Alpha[i]:
                CO2_pen += omega_vekting[omega]*model.CO2_Penalty_Cost_SP[omega,i].value
    print("CO2 Pen in euro=",CO2_pen)
    
    for omega in model.omega:
        for i in model.i:
            for t in model.t:
                if 0<Alpha[i]:
                    SO_pen += omega_vekting[i]*(model.Cost_P_Res_Artificial_SP_total[omega,i,t].value+model.Cost_P_Res_Art_TooMuch_SP[omega,i,t].value)
                
                
    print("System operator penalty=",SO_pen)
    
    soc_depart = {}
    
    for omega in model.omega:
        for i in model.i:
            for t in model.t:
                if t!=model.t.first():
                    soc_depart[omega,i,t] = model.soc_dep_SP[omega,i,t].value
                

    
    soc_pen_agg_w = 0
    teller =0
    
    for omega in model.omega:
        for i in model.i:
            for t in model.t:
                if 0<Alpha[i]:
                    if t != model.t.first():
                        if 0 < soc_depart[omega,i,t]:
                            if model.P_Res_Alpha_first<= model.P_Res_Alpha_second:
                                soc_pen_agg_w += model.pen_min_soc_SP[omega,i,t].value/model.pen_limit_soc_min_SP[omega,model.t.prev(t)]
                                teller += 1
                            elif model.P_Res_Alpha_second < model.P_Res_Alpha_first:
                                soc_pen_agg_w += model.pen_max_soc_SP[omega,i,t].value/model.pen_limit_soc_max_SP[omega,model.t.prev(t)]
                                teller += 1
    
    
    if model.P_Res_Alpha_first<= model.P_Res_Alpha_second:
        if soc_dep_requirement ==0.9:
            soc_pen_agg_w =90 -(soc_pen_agg_w/teller)*40
        elif soc_dep_requirement ==0.8:
            soc_pen_agg_w =80 -(soc_pen_agg_w/teller)*30
        elif soc_dep_requirement ==0.7:
            soc_pen_agg_w =70 -(soc_pen_agg_w/teller)*20
    else:
        if soc_dep_requirement ==0.9:
            soc_pen_agg_w =90 + (soc_pen_agg_w/teller)*40
        elif soc_dep_requirement ==0.8:
            soc_pen_agg_w = 80+  (soc_pen_agg_w/teller)*30
        elif soc_dep_requirement ==0.7:
            soc_pen_agg_w = 70+ (soc_pen_agg_w/teller)*20
        
    
    
    print("SOC Pen Percentage =", soc_pen_agg_w)
    
    
    """
    for omega in model.omega:
        for i in model.i:
            for t in model.t:
                if 0<Alpha[i]:
                    SOC_pen += 

    """
    
    """
    This parts saves the results to an excel-file SCENARIO BY SCENARIO
    """
    
    
    """
    output_SP = {}
    output_SP['Objective'] = {}
    output_SP['Income_Res_Act_SP'] = {}
    output_SP['Cost_Deg_Res_SP'] = {}
    output_SP['Cost_Unavailable_SP'] = {}
    output_SP['Cost_Res_Tariff_SP'] = {}
    output_SP['soc_SP'] = {}
    output_SP['soc_battery_SP'] = {}
    output_SP['Cost_P_Res_Artificial_SP_total'] = {}
    output_SP['P_Res_Artificial_SP_total'] ={}
    output_SP['P_Res_Artificial_SP_2B'] ={}
    output_SP['P_Res_Artificial_SP_2PL'] ={}
    output_SP['P_En_PL2G_SP'] = {}
    output_SP['P_En_G2PL_SP'] = {}
    output_SP['P_En_B2G_SP'] = {}
    output_SP['P_En_G2B_SP'] = {}
    output_SP['Income_CO2_SP'] = {}
    output_SP['CO2_Penalty_SP'] = {}
    output_SP['CO2_Penalty_Cost_SP'] = {}
    output_SP['Income_En_PL2G_SP'] = {}
    output_SP['Income_Cap_Res_SP'] = {}
    output_SP['Income_Inc'] = {}
    output_SP['Cost_En_G2PL_SP'] = {}
    output_SP['Cost_soc_min_SP'] = {}
    


    
    
    

    for omega in model.omega:
        for i in model.i:
            for t in model.t:
            
                if 0 < Alpha[i]:
                    output_SP['Objective'] = model.objective_SP()
                    output_SP['Income_Res_Act_SP'][omega,i,t] = model.Income_Res_Act_SP[omega,i,t].value
                    output_SP['Cost_Deg_Res_SP'][omega,i,t] = model.Cost_Deg_Res_SP[omega,i,t].value
                    output_SP['Cost_Unavailable_SP'][omega,i,t] = model.Cost_Unavailable_SP[omega,i,t].value
                    output_SP['Cost_Res_Tariff_SP'][omega,i,t]= model.Cost_Res_Tariff_SP[omega,i,t].value
                    output_SP['soc_SP'][omega,i,t]= model.soc_SP[omega,i,t].value
                    output_SP['soc_battery_SP'][omega,i,t] = model.soc_battery_SP[omega,i,t].value
                    output_SP['Cost_P_Res_Artificial_SP_total'][omega,i,t] = model.Cost_P_Res_Artificial_SP_total[omega,i,t].value
                    output_SP['P_Res_Artificial_SP_total'][omega,i,t]= model.P_Res_Artificial_SP_total[omega,i,t].value
                    output_SP['P_Res_Artificial_SP_2B'][omega,i,t]= model.P_Res_Artificial_SP_2B[omega,i,t].value
                    output_SP['P_Res_Artificial_SP_2PL'][omega,i,t] = model.P_Res_Artificial_SP_2PL[omega,i,t].value
                    output_SP['P_En_PL2G_SP'][omega,i,t] = model.P_En_PL2G_SP[omega,i,t].value
                    output_SP['P_En_G2PL_SP'][omega,i,t]= model.P_En_G2PL_SP[omega,i,t].value
                    output_SP['P_En_B2G_SP'][omega,i,t] = model.P_En_B2G_SP[omega,i,t].value
                    output_SP['P_En_G2B_SP'][omega,i,t] = model.P_En_G2B_SP[omega,i,t].value
                    output_SP['Income_CO2_SP'][omega,i,t]= model.Income_CO2_SP[omega,i,t].value
                    output_SP['CO2_Penalty_SP'][omega,i]= model.CO2_Penalty_SP[omega,i].value
                    output_SP['CO2_Penalty_Cost_SP'][omega,i]= model.CO2_Penalty_Cost_SP[omega,i].value
                    
                    output_SP['Income_En_PL2G_SP'][omega,i,t] = model.Income_En_PL2G_SP[omega,i,t].value
                    output_SP['Income_Cap_Res_SP'][omega,i,t] = model.Income_Cap_Res_SP[omega,i,t].value
                    output_SP['Cost_En_G2PL_SP'][omega,i,t] = model.Cost_En_G2PL_SP[omega,i,t].value
                    output_SP['Cost_soc_min_SP'][omega,i,t] = model.Cost_soc_min_SP[omega,i,t].value
      
        
    resultater_SP = pd.DataFrame(data = output_SP) 

    
    
    #Store each result in an excel file, given a separate sheet
    with pd.ExcelWriter("Results_2step.xlsx") as writer:
        resultater_SP['Objective'].to_excel(writer, sheet_name = "Obj_function")
        resultater_SP['Income_Res_Act_SP'].to_excel(writer, sheet_name = "Income_Res_Act")
        resultater_SP['Cost_Deg_Res_SP'].to_excel(writer, sheet_name = "Cost_Deg_Res")
        resultater_SP['Cost_Unavailable_SP'].to_excel(writer, sheet_name = "Cost_Unavailable")
        resultater_SP['Cost_Res_Tariff_SP'].to_excel(writer, sheet_name = "Cost_Res_Tariff")
        resultater_SP['soc_SP'].to_excel(writer, sheet_name = "soc")
        resultater_SP['soc_battery_SP'].to_excel(writer, sheet_name = "soc_battery")
        resultater_SP['Cost_P_Res_Artificial_SP_total'].to_excel(writer, sheet_name = "Cost_P_Res_Artificial_SP_total")
        resultater_SP['P_Res_Artificial_SP_total'].to_excel(writer, sheet_name = "P_Res_Artificial_SP_total")
        resultater_SP['P_Res_Artificial_SP_2B'].to_excel(writer, sheet_name = "P_Res_Artificial_SP_2B")
        resultater_SP['P_Res_Artificial_SP_2PL'].to_excel(writer, sheet_name = "P_Res_Artificial_SP_2PL")
        resultater_SP['P_En_PL2G_SP'].to_excel(writer, sheet_name = "P_En_PL2G_SP")
        resultater_SP['P_En_G2PL_SP'].to_excel(writer, sheet_name = "P_En_G2PL_SP")
        resultater_SP['P_En_B2G_SP'].to_excel(writer, sheet_name = "P_En_B2G_SP")
        resultater_SP['P_En_G2B_SP'].to_excel(writer, sheet_name = "P_En_G2B_SP")
        resultater_SP['Income_CO2_SP'].to_excel(writer, sheet_name = "Income_CO2_SP")
        resultater_SP['CO2_Penalty_SP'].to_excel(writer, sheet_name = "CO2_Penalty_SP")
        resultater_SP['CO2_Penalty_Cost_SP'].to_excel(writer, sheet_name = "CO2_Penalty_Cost_SP")
        
        resultater_SP['Income_En_PL2G_SP'].to_excel(writer, sheet_name = "Income_En_PL2G_SP")
        resultater_SP['Income_Cap_Res_SP'].to_excel(writer, sheet_name = "Income_Cap_Res_SP")
        resultater_SP['Cost_En_G2PL_SP'].to_excel(writer, sheet_name = "Cost_En_G2PL_SP")
        resultater_SP['Cost_soc_min_SP'].to_excel(writer, sheet_name = "Cost_soc_min_SP")

    






    



    

    """
    """
    This parts saves the results to an excel-file SCENARIOS AGGREGATED
    """
    

    
    
    output_SP = {}
    output_SP['Objective'] = {}
    output_SP['Income_Res_Act_SP'] = {}
    output_SP['Cost_Deg_Res_SP'] = {}
    output_SP['Cost_Unavailable_SP'] = {}
    output_SP['Cost_Res_Tariff_SP'] = {}
    output_SP['soc_SP'] = {}
    output_SP['soc_battery_SP'] = {}
    output_SP['Cost_P_Res_Artificial_SP_total'] = {}
    output_SP['P_Res_Artificial_SP_total'] ={}
    output_SP['P_Res_Artificial_SP_2B'] ={}
    output_SP['P_Res_Artificial_SP_2PL'] ={}
    output_SP['P_En_PL2G_SP'] = {}
    output_SP['P_En_G2PL_SP'] = {}
    output_SP['P_En_B2G_SP'] = {}
    output_SP['P_En_G2B_SP'] = {}
    output_SP['Income_CO2_SP'] = {}
    output_SP['CO2_Penalty_SP'] = {}
    output_SP['CO2_Penalty_Cost_SP'] = {}
    
    output_SP['P_Res_Act_SP'] = {}
    output_SP['P_Res_Act_SP_PL'] = {}
    output_SP['P_Res_Act_SP_B'] = {}
    output_SP['soc_dep_SP'] = {}
    output_SP['Cost_Deg_En_SP'] = {}
    output_SP['Cost_Battery_Deg_SP'] = {}
    #output_SP['Cost_soc_min_SP'] = {}
    #output_SP['pen_min_soc_SP'] = {}
    output_SP['P_Res_Artificial_SP_PL_TooMuch'] = {}
    output_SP['P_Res_Artificial_SP_B_TooMuch'] = {}
    output_SP['P_Res_Artificial_SP_Total_TooMuch'] = {}
    output_SP['Cost_P_Res_Art_TooMuch_SP'] = {}
    output_SP['pen_max_soc_SP'] = {}
    output_SP['pen_min_soc_SP'] = {}
    
    
    for i in model.i:
        for t in model.t:
                if 0 < Alpha[i]:
                    #output_SP['Income_Res_Act_SP'][i,t] = 0
                    output_SP['Cost_Deg_Res_SP'][i,t] = 0
                    output_SP['Cost_Unavailable_SP'][i,t] = 0
                    #output_SP['Cost_Res_Tariff_SP'][i,t] = 0
                    output_SP['soc_SP'][i,t] = 0
                    output_SP['soc_battery_SP'][i,t] = 0
                    output_SP['Cost_P_Res_Artificial_SP_total'][i,t] = 0
                    output_SP['P_Res_Artificial_SP_total'][i,t] =0
                    output_SP['P_Res_Artificial_SP_2B'][i,t] = 0
                    output_SP['P_Res_Artificial_SP_2PL'][i,t] = 0
                    output_SP['P_En_PL2G_SP'][i,t] = 0
                    output_SP['P_En_G2PL_SP'][i,t] = 0
                    output_SP['P_En_B2G_SP'][i,t] = 0
                    output_SP['P_En_G2B_SP'][i,t] = 0
                    output_SP['Income_CO2_SP'][i,t]= 0
                    output_SP['CO2_Penalty_SP'][i]= 0
                    output_SP['CO2_Penalty_Cost_SP'][i]= 0
                    
                    output_SP['P_Res_Act_SP'][i,t] = 0
                    output_SP['P_Res_Act_SP_PL'][i,t] = 0
                    output_SP['P_Res_Act_SP_B'][i,t] = 0
                    output_SP['soc_dep_SP'][i,t] = 0
                    output_SP['Cost_Deg_En_SP'][i,t] = 0
                    output_SP['Cost_Battery_Deg_SP'][i,t] = 0
                    #output_SP['Cost_soc_min_SP'][i,t] = 0
                    #output_SP['pen_min_soc_SP'][i,t] = 0
                    output_SP['P_Res_Artificial_SP_PL_TooMuch'][i,t] = 0
                    output_SP['P_Res_Artificial_SP_B_TooMuch'][i,t] = 0
                    output_SP['P_Res_Artificial_SP_Total_TooMuch'][i,t] = 0
                    output_SP['Cost_P_Res_Art_TooMuch_SP'][i,t] = 0
                    output_SP['pen_max_soc_SP'][i,t] = 0
                    output_SP['pen_min_soc_SP'][i,t] = 0

    
    
    

    for omega in model.omega:
        for i in model.i:
            for t in model.t:
            
                if 0 < Alpha[i]:
                    output_SP['Objective'] = model.objective_SP()
                    output_SP['Income_Res_Act_SP'][omega,i,t] =model.Income_Res_Act_SP[omega,i,t].value
                    output_SP['Cost_Deg_Res_SP'][i,t] += omega_vekting[omega]*model.Cost_Deg_Res_SP[omega,i,t].value
                    output_SP['Cost_Unavailable_SP'][i,t] += omega_vekting[omega]*model.Cost_Unavailable_SP[omega,i,t].value
                    #output_SP['Cost_Res_Tariff_SP'][i,t] += omega_vekting[omega]*model.Cost_Res_Tariff_SP[omega,i,t].value
                    output_SP['soc_SP'][i,t] += omega_vekting[omega]*model.soc_SP[omega,i,t].value
                    output_SP['soc_battery_SP'][i,t] += omega_vekting[omega]*model.soc_battery_SP[omega,i,t].value
                    output_SP['Cost_P_Res_Artificial_SP_total'][i,t] += omega_vekting[omega]*model.Cost_P_Res_Artificial_SP_total[omega,i,t].value
                    output_SP['P_Res_Artificial_SP_total'][i,t] += omega_vekting[omega]*model.P_Res_Artificial_SP_total[omega,i,t].value
                    output_SP['P_Res_Artificial_SP_2B'][i,t]+= omega_vekting[omega]*model.P_Res_Artificial_SP_2B[omega,i,t].value
                    output_SP['P_Res_Artificial_SP_2PL'][i,t] += omega_vekting[omega]*model.P_Res_Artificial_SP_2PL[omega,i,t].value
                    output_SP['P_En_PL2G_SP'][i,t] += omega_vekting[omega]*model.P_En_PL2G_SP[omega,i,t].value
                    output_SP['P_En_G2PL_SP'][i,t] += omega_vekting[omega]*model.P_En_G2PL_SP[omega,i,t].value
                    output_SP['P_En_B2G_SP'][i,t] += omega_vekting[omega]*model.P_En_B2G_SP[omega,i,t].value
                    output_SP['P_En_G2B_SP'][i,t] += omega_vekting[omega]*model.P_En_G2B_SP[omega,i,t].value
                    output_SP['Income_CO2_SP'][i,t]+= omega_vekting[omega]*model.Income_CO2_SP[omega,i,t].value

                    
                    output_SP['P_Res_Act_SP'][i,t] += omega_vekting[omega]*model.P_Res_Act_SP[omega,i,t]
                    output_SP['P_Res_Act_SP_PL'][i,t] += omega_vekting[omega]*model.P_Res_Act_SP_PL[omega,i,t].value
                    output_SP['P_Res_Act_SP_B'][i,t] += omega_vekting[omega]*model.P_Res_Act_SP_B[omega,i,t].value
                    output_SP['soc_dep_SP'][i,t] += omega_vekting[omega]*model.soc_dep_SP[omega,i,t].value
                    output_SP['Cost_Deg_En_SP'][i,t] += omega_vekting[omega]*model.Cost_Deg_En_SP[omega,i,t].value
                    output_SP['Cost_Battery_Deg_SP'][i,t] += omega_vekting[omega]*model.Cost_Battery_Deg_SP[omega,i,t].value
                    #output_SP['Cost_soc_min_SP'][i,t] += omega_vekting[omega]*model.Cost_soc_min_SP[omega,i,t].value
                    #output_SP['pen_min_soc_SP'][i,t] += omega_vekting[omega]*model.pen_min_soc_SP[omega,i,t].value
                    output_SP['P_Res_Artificial_SP_PL_TooMuch'][i,t] += omega_vekting[omega]*model.P_Res_Artificial_SP_PL_TooMuch[omega,i,t].value
                    output_SP['P_Res_Artificial_SP_B_TooMuch'][i,t] += omega_vekting[omega]*model.P_Res_Artificial_SP_B_TooMuch[omega,i,t].value
                    output_SP['P_Res_Artificial_SP_Total_TooMuch'][i,t] += omega_vekting[omega]*model.P_Res_Artificial_SP_Total_TooMuch[omega,i,t].value
                    output_SP['Cost_P_Res_Art_TooMuch_SP'][i,t] += omega_vekting[omega]*model.Cost_P_Res_Art_TooMuch_SP[omega,i,t].value
                    
                    output_SP['pen_max_soc_SP'][i,t] += omega_vekting[omega]*model.pen_max_soc_SP[omega,i,t].value
                    output_SP['pen_min_soc_SP'][i,t] += omega_vekting[omega]*model.pen_min_soc_SP[omega,i,t].value
              
                
    for i in model.i:
        for omega in model.omega:
            if 0 < Alpha[i]:
                output_SP['CO2_Penalty_SP'][i]+= omega_vekting[omega]*model.CO2_Penalty_SP[omega,i].value
                output_SP['CO2_Penalty_Cost_SP'][i]+= omega_vekting[omega]*model.CO2_Penalty_Cost_SP[omega,i].value
        
    resultater_SP = pd.DataFrame(data = output_SP) 

    
    
    #Store each result in an excel file, given a separate sheet
    with pd.ExcelWriter("Results_2step_NewTime_TOU.xlsx") as writer:
        resultater_SP['Objective'].to_excel(writer, sheet_name = "Obj_function")
        resultater_SP['Income_Res_Act_SP'].to_excel(writer, sheet_name = "Income_Res_Act")
        resultater_SP['Cost_Deg_Res_SP'].to_excel(writer, sheet_name = "Cost_Deg_Res")
        resultater_SP['Cost_Unavailable_SP'].to_excel(writer, sheet_name = "Cost_Unavailable")
        #resultater_SP['Cost_Res_Tariff_SP'].to_excel(writer, sheet_name = "Cost_Res_Tariff")
        resultater_SP['soc_SP'].to_excel(writer, sheet_name = "soc")
        resultater_SP['soc_battery_SP'].to_excel(writer, sheet_name = "soc_battery")
        resultater_SP['Cost_P_Res_Artificial_SP_total'].to_excel(writer, sheet_name = "Cost_P_Res_Artificial_SP_total")
        resultater_SP['P_Res_Artificial_SP_total'].to_excel(writer, sheet_name = "P_Res_Artificial_SP_total")
        resultater_SP['P_Res_Artificial_SP_2B'].to_excel(writer, sheet_name = "P_Res_Artificial_SP_2B")
        resultater_SP['P_Res_Artificial_SP_2PL'].to_excel(writer, sheet_name = "P_Res_Artificial_SP_2PL")
        resultater_SP['P_En_PL2G_SP'].to_excel(writer, sheet_name = "P_En_PL2G_SP")
        resultater_SP['P_En_G2PL_SP'].to_excel(writer, sheet_name = "P_En_G2PL_SP")
        resultater_SP['P_En_B2G_SP'].to_excel(writer, sheet_name = "P_En_B2G_SP")
        resultater_SP['P_En_G2B_SP'].to_excel(writer, sheet_name = "P_En_G2B_SP")
        resultater_SP['Income_CO2_SP'].to_excel(writer, sheet_name = "Income_CO2_SP")
        resultater_SP['CO2_Penalty_SP'].to_excel(writer, sheet_name = "CO2_Penalty_SP")
        resultater_SP['CO2_Penalty_Cost_SP'].to_excel(writer, sheet_name = "CO2_Penalty_Cost_SP")
        
        resultater_SP['P_Res_Act_SP'].to_excel(writer, sheet_name = "P_Res_Act_SP")
        resultater_SP['P_Res_Act_SP_PL'].to_excel(writer, sheet_name = "P_Res_Act_SP_PL")
        resultater_SP['P_Res_Act_SP_B'].to_excel(writer, sheet_name = "P_Res_Act_SP_B")
        resultater_SP['soc_dep_SP'].to_excel(writer, sheet_name = "soc_dep_SP")
        resultater_SP['Cost_Deg_En_SP'].to_excel(writer, sheet_name = "Cost_Deg_En_SP")
        resultater_SP['Cost_Battery_Deg_SP'].to_excel(writer, sheet_name = "Cost_Battery_Deg_SP")
        #resultater_SP['Cost_soc_min_SP'].to_excel(writer, sheet_name = "Cost_soc_min_SP")
        #resultater_SP['pen_min_soc_SP'].to_excel(writer, sheet_name = "pen_min_soc_SP")
        resultater_SP['P_Res_Artificial_SP_PL_TooMuch'].to_excel(writer, sheet_name = "P_Res_Artificial_SP_PL_TooMuch")
        resultater_SP['P_Res_Artificial_SP_B_TooMuch'].to_excel(writer, sheet_name = "P_Res_Artificial_SP_B_TooMuch")
        resultater_SP['P_Res_Artificial_SP_Total_TooMuch'].to_excel(writer, sheet_name = "P_Res_Art_SP_Tot_TooMuch")
        resultater_SP['Cost_P_Res_Art_TooMuch_SP'].to_excel(writer, sheet_name = "Cost_P_Res_Art_TooMuch_SP")
        resultater_SP['pen_max_soc_SP'].to_excel(writer, sheet_name = "pen_max_soc_SP")
        resultater_SP['pen_min_soc_SP'].to_excel(writer, sheet_name = "pen_min_soc_SP")


        
    return(model)
    
model = Optimization_method()






