#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep  9 10:31:43 2019

@author: thomaskallevik
"""

import numpy as np
import sys
import time
import pandas as pd
import pyomo.environ as pyo
from pyomo.opt import SolverFactory
import matplotlib.pyplot as plt

def Read_from_excel(file):
    
    data = {}
    
    Excel_sheets = {'Parameters','FOR_PL','soc_PEV','Energy_prices','Input_prob_funct','Cap_PEV','Agg_Perc', 'Energy_prices_PBDRP','Inc_Prices', 'P_Cont','Pen_Prices','solar_radiation', 'Scenario_00', 'Scenario_01', 'Scenario_02', 'Scenario_03', 'Scenario_04', 'Scenario_05', 'Scenario_06', 'Scenario_07', 'Scenario_08', 'Scenario_09','P_initial_00','P_initial_01','P_initial_02','P_initial_03','P_initial_04','P_initial_05','P_initial_06','P_initial_07','P_initial_08','P_initial_09', 'soc_dep','CO2','CapacityPrice','Fixed_Price_Tariff'}
    Data_names = {'Parameters':'Parameter','FOR_PL':'FOR_Penalty','soc_PEV':'SOC_PEV','Energy_prices':'Energy_Price','Input_prob_funct':'Prob_functions','Cap_PEV':'Cap_PEV','Agg_Perc':'Aggregated_Percentage', 'P_initial':'P_ini', 'Energy_prices_PBDRP':'En_Price_DRP','Inc_Prices':'Incentive_Price', 'P_Cont':'P_Contract', 'Pen_Prices':'Penalty','solar_radiation':'Solar_Radiation', 'Scenario_00':'0', 'Scenario_01':'1', 'Scenario_02':'2', 'Scenario_03':'3', 'Scenario_04':'4', 'Scenario_05':'5', 'Scenario_06':'6', 'Scenario_07':'7', 'Scenario_08':'8', 'Scenario_09':'9', 'P_initial_00':'P_ini0', 'P_initial_01':'P_ini1', 'P_initial_02':'P_ini2', 'P_initial_03':'P_ini3', 'P_initial_04':'P_ini4', 'P_initial_05':'P_ini5', 'P_initial_06':'P_ini6', 'P_initial_07':'P_ini7', 'P_initial_08':'P_ini8', 'P_initial_09':'P_ini9', 'soc_dep':'soc_departure','CO2':'CO2_power','CapacityPrice':'Capacity_Price','Fixed_Price_Tariff':'Fixed_Tariff'}
    Num_Names = {'Parameters':'NumParam','FOR_PL':'NumFOR_PL','soc_PEV':'Numsoc_Pev','Energy_prices':'NumEnergyPrice','Input_prob_funct':'NumProbability','Cap_PEV':'NumCapPEV','Agg_Perc':'NumAggPerc', 'Energy_prices_PBDRP':'NumEn_Price_DRP','Inc_Prices':'NumIncentive_Price', 'P_Cont':'NumP_Contract', 'Pen_Prices':'NumPenalty','solar_radiation':'NumSolar_Radiation', 'Scenario_00':'Num0', 'Scenario_01':'Num1', 'Scenario_02':'Num2', 'Scenario_03':'Num3', 'Scenario_04':'Num4', 'Scenario_05':'Num5', 'Scenario_06':'Num6', 'Scenario_07':'Num7', 'Scenario_08':'Num8', 'Scenario_09':'Num9', 'P_initial_00':'Num0', 'P_initial_01':'Num1', 'P_initial_02':'Num2', 'P_initial_03':'Num3', 'P_initial_04':'Num4', 'P_initial_05':'Num5', 'P_initial_06':'Num6', 'P_initial_07':'Num7', 'P_initial_08':'Num8', 'P_initial_09':'Num9', 'soc_dep':'Numsoc_departure','CO2':'NumCO2_power','CapacityPrice':'NumCapacity_Price','Fixed_Price_Tariff':'NumFixed_Tariff'}
    List_Names = {'Parameters':'ListParam','FOR_PL':'ListFOR_PL','soc_PEV':'Listsoc_Pev','Energy_prices':'ListEnergyPrice','Input_prob_funct':'ListProbability','Cap_PEV':'ListCapPEV','Agg_Perc':'ListAggPerc', 'Energy_prices_PBDRP':'LenEn_Price_DRP','Inc_Prices':'LenIncentive_Price', 'P_Cont':'LenP_Contract', 'Pen_Prices':'LenPenalty','solar_radiation':'LenSolar_Radiation', 'Scenario_00':'Len0', 'Scenario_01':'Len1', 'Scenario_02':'Len2', 'Scenario_03':'Len3', 'Scenario_04':'Len4', 'Scenario_05':'Len5', 'Scenario_06':'Len6', 'Scenario_07':'Len7', 'Scenario_08':'Len8', 'Scenario_09':'Len9', 'P_initial_00':'Len0', 'P_initial_00':'Len0', 'P_initial_01':'Len1', 'P_initial_02':'Len2', 'P_initial_03':'Len3', 'P_initial_04':'Len4', 'P_initial_05':'Len5', 'P_initial_06':'Len6', 'P_initial_07':'Len7', 'P_initial_08':'Len8', 'P_initial_09':'Len9', 'soc_dep':'Listsoc_departure','CO2':'ListCO2_power','CapacityPrice':'ListCapacity_Price','Fixed_Price_Tariff':'ListFixed_Tariff'}
    
    for sheet in Excel_sheets:      #For each sheet
        df      = pd.read_excel(file, sheet_name = sheet)         #Read sheet, exclude title
        
        df      = df.set_index(df.columns[0])                                   #Set first column as index
        num     = len(df.loc[:])                                                #Find length of dataframe
        df      = df.to_dict()
        
        
        
        df[Num_Names[sheet]]        = num                                       #Store length of dataframe in dictionary
        df[List_Names[sheet]]       = np.arange(1,num+1)
        
        data[Data_names[sheet]]     = df                                        #Store dataframe in dictionary
        
    return(data)  

data = Read_from_excel('Input_data_Base.xlsx')